﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				ApplicationMenu.cs
** description:	This file contains the application menu event handlers.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Printing;
using System.Windows.Forms;
using RibbonLib;
using RibbonLib.Controls;
using RibbonLib.Controls.Events;
using RibbonLib.Interop;
using TXTextControl;


namespace TX_Text_Control_Words
{
   public partial class FormMain : Form
   {
      #region "  Private fields  "

      private RibbonRecentItems _ribbonRecentItems;
      private List<RecentItemsPropertySet> _recentItemsList;
      private RibbonButton _btnExit;
      private RibbonSplitButtonGallery _spltBtnGlrySaveAs;
      private RibbonButton[] _btnsGallerySaveAs;
      private RibbonSplitButtonGallery _spltBtnGlryPrint;
      private RibbonButton[] _btnsGalleryPrint;
      private RibbonButton _btnNew;
      private RibbonButton _btnOpen;
      private RibbonButton _btnSave;
      private RibbonButton _btnOptions;
      private RibbonButton _btnPrint;
      private RibbonButton _btnUndo;
      private RibbonButton _btnRedo;
      private RibbonDropDownGallery _drpDnGlryLoadTemplate;
      private RibbonButton[] _btnsGalleryLoadTemplate;

      private static string[] SaveAsButtonLabelText = 
      {
         "Word 97 – 2003 Document (*.doc)",
         "Word Document (*.docx)",
         "Rich Text Format (*.rtf)",
         "Adobe PDF Document (*.pdf)",
         "Hypertext Markup Language (*.html)",
         "Other Formats",
      };

      private static string[] PrintButtonLabelText =
      {
         "Print",
         "Quick Print",
         "Print Preview"
      };

      private static string[] LoadTemplateButtonLabelText =
      {
         "Load a Sample Invoice.",
         "Load a Sample Quote.",
         "Load a Sample Letter."
      };

      #endregion // Private fields

      #region "  Enumerations  "

      enum RibbonMarkupCommand_ApplicationMenu : uint
      {
         cmdApplicationMenu = 5000,
         cmdBtnNew = 5001,
         cmdBtnOpen = 5002,
         cmdBtnSave = 5003,
         cmdGlrySaveAs = 5004,
         cmdGlryPrint = 5005,
         cmdBtnOptions = 5006,
         cmdBtnExit = 5007,
         cmdBtnPrint = 5008,
         cmdMRUItems = 5009,
         cmdBtnUndo = 5010,
         cmdBtnRedo = 5011,
         cmdGlryLoadTemplate = 5012,
      }

      enum SaveAsButtonCommand : uint
      {
         cmdBtnSaveAsDoc = DynamicCommandIdStart + 40,
         cmdBtnSaveAsDocx,
         cmdBtnSaveAsRTF,
         cmdBtnSaveAsPDF,
         cmdBtnSaveAsHTML,
         cmdBtnSaveAsOtherFormats,
      }

      enum PrintButtonCommand : uint
      {
         cmdBtnPrint = DynamicCommandIdStart + 50,
         cmdBtnPrintQuick,
         cmdBtnPrintPreview,
      }

      enum LoadTemplateButtonCommand : uint
      {
         cmdBtnLoadTemplate_Invoice = DynamicCommandIdStart + 60,
         cmdBtnLoadTemplate_Quote,
         cmdBtnLoadTemplate_Letter,
      }

      #endregion // Enumerations

      #region "  Ribbon event handlers  "

      void RecentItems_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (e.Key.PropertyKey == RibbonProperties.RecentItems)
         {
            object[] objectArray = (object[])e.CurrentValue.PropVariant.Value;
            for (int i = 0; i < objectArray.Length; ++i)
            {
               IUISimplePropertySet propertySet = objectArray[i] as IUISimplePropertySet;
               if (propertySet != null)
               {
                  PropVariant propLabel;
                  propertySet.GetValue(ref RibbonProperties.Label, out propLabel);
                  string strLabel = (string)propLabel.Value;

                  PropVariant propPinned;
                  propertySet.GetValue(ref RibbonProperties.Pinned, out propPinned);
                  bool bPinned = (bool)propPinned.Value;

                  _recentItemsList[i].Pinned = bPinned;  // Update pinned state
               }
            }
         }
         else if (e.Key.PropertyKey == RibbonProperties.SelectedItem)
         {
            uint selectedItem = (uint)e.CurrentValue.PropVariant.Value;
            _fileHandler.OpenRecentFile((int)selectedItem);
         }
      }

      void BtnExit_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         this.BeginInvoke(new MethodInvoker(Close));
      }

      void SplitBtnGallerySaveAs_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _fileHandler.FileSaveAs(TXTextControl.StreamType.RichTextFormat);
      }

      void BtnGlrySaveAs_ItemsSourceReady(object sender, EventArgs e)
      {
         // Create buttons and images

         _btnsGallerySaveAs = new RibbonButton[_imgLstSaveAs.Images.Count];
         Array commandIds = Enum.GetValues(typeof(SaveAsButtonCommand));

         for (uint i = 0; i < _btnsGallerySaveAs.Length; ++i)
         {
            _btnsGallerySaveAs[i] = new RibbonButton(_ribbonMain, (uint)commandIds.GetValue(i))
            {
               Label = SaveAsButtonLabelText[(int)i],
               LargeImage = _ribbonMain.ConvertToUIImage((Bitmap)_imgLstSaveAs.Images[(int)i])
            };
            _btnsGallerySaveAs[i].ExecuteEvent += BtnsGallerySaveAs_ExecuteEvent;
         }

         // Add button commands

         IUICollection itemsSource = _spltBtnGlrySaveAs.ItemsSource;
         itemsSource.Clear();

         foreach (object n in commandIds)
         {
            itemsSource.Add(new GalleryCommandPropertySet()
            {
               CommandID = (uint)n,
               CommandType = CommandType.Action,
            });
         }
      }

      void BtnsGallerySaveAs_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         TXTextControl.StreamType streamType = TXTextControl.StreamType.RichTextFormat;

         RibbonButton btn = sender as RibbonButton;
         if (btn == null) return;

         var cmd = (SaveAsButtonCommand)btn.CommandID;

         switch (cmd)
         {
            case SaveAsButtonCommand.cmdBtnSaveAsDoc:
               streamType = TXTextControl.StreamType.MSWord;
               break;

            case SaveAsButtonCommand.cmdBtnSaveAsDocx:
               streamType = TXTextControl.StreamType.WordprocessingML;
               break;

            case SaveAsButtonCommand.cmdBtnSaveAsRTF:
               streamType = TXTextControl.StreamType.RichTextFormat;
               break;

            case SaveAsButtonCommand.cmdBtnSaveAsPDF:
               streamType = TXTextControl.StreamType.AdobePDF;
               break;

            case SaveAsButtonCommand.cmdBtnSaveAsHTML:
               streamType = TXTextControl.StreamType.HTMLFormat;
               break;
         }

         try
         {
            _fileHandler.FileSaveAs(streamType);
         }
         catch (Exception exc)
         {
            MessageBox.Show(exc.Message, "Error saving document", MessageBoxButtons.OK, MessageBoxIcon.Error);
         }
      }

      void SplitBtnGalleryPrint_ItemsSourceReady(object sender, EventArgs e)
      {
         // Create buttons and images

         _btnsGalleryPrint = new RibbonButton[_imgLstPrint.Images.Count];
         Array commandIds = Enum.GetValues(typeof(PrintButtonCommand));

         for (uint i = 0; i < _btnsGalleryPrint.Length; ++i)
         {
            _btnsGalleryPrint[i] = new RibbonButton(_ribbonMain, (uint)commandIds.GetValue(i))
            {
               Label = PrintButtonLabelText[(int)i],
               LargeImage = _ribbonMain.ConvertToUIImage((Bitmap)_imgLstPrint.Images[(int)i])
            };
            _btnsGalleryPrint[i].ExecuteEvent += BtnsGalleryPrint_ExecuteEvent;
         }

         // Add button commands

         IUICollection itemsSource = _spltBtnGlryPrint.ItemsSource;
         itemsSource.Clear();

         foreach (object n in commandIds)
         {
            itemsSource.Add(new GalleryCommandPropertySet()
            {
               CommandID = (uint)n,
               CommandType = CommandType.Action,
            });
         }
      }

      void BtnsGalleryPrint_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         RibbonButton btn = sender as RibbonButton;
         if (btn == null) return;

         var cmd = (PrintButtonCommand)btn.CommandID;

         switch (cmd)
         {
            case PrintButtonCommand.cmdBtnPrint:
               BtnPrint_ExecuteEvent(sender, e);
               break;

            case PrintButtonCommand.cmdBtnPrintPreview:
               _textControl.PrintPreview(_fileHandler.PrintDocumentName);
               break;

            case PrintButtonCommand.cmdBtnPrintQuick:
               _textControl.Print(new PrintDocument()
               {
                  PrinterSettings = new PrinterSettings()
                  {
                     FromPage = 1,
                     ToPage = _textControl.Pages,
                     Copies = 1,
                     Collate = true,
                  },
               });
               break;
         }
      }

      void BtnPrint_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Print(_fileHandler.PrintDocumentName);
      }

      void BtnOptions_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         (new FrmOptions(_textControl) { FileHandler = _fileHandler }).ShowDialog();
      }

      void BtnSave_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         try
         {
            _fileHandler.FileSave();
         }
         catch (Exception exc)
         {
            MessageBox.Show(exc.Message, "Error saving document", MessageBoxButtons.OK, MessageBoxIcon.Error);
         }
      }

      void BtnOpen_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (_fileHandler.DocumentDirty && !FileSaveQuestion()) return;

         _fileHandler.DocumentFileName = string.Empty;
         _fileHandler.FileOpen();

         // (Re-) Set field display mode to default value
         ToggleFieldDisplayModeButton(FieldDisplayMode.ShowFieldText);
         _tglBtnPreviewFields.BooleanValue = false;
      }

      void BtnNew_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (_fileHandler.DocumentDirty && !FileSaveQuestion()) return;

         _fileHandler.DocumentFileName = string.Empty;
         _textControl.ResetContents();
         Text = _fileHandler.PrintDocumentName;
         _fileHandler.DocumentDirty = false;

         // (Re-) Set field display mode to default value
         ToggleFieldDisplayModeButton(FieldDisplayMode.ShowFieldText);
         _tglBtnPreviewFields.BooleanValue = false;

         // Update styles list
         FormattingStyleListChanged();
      }

      void BtnRedo_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Redo();
      }

      void BtnUndo_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Undo();
      }

      void DrpDnGlryLoadTemplate_ItemsSourceReady(object sender, EventArgs e)
      {
         // Create buttons and images

         _btnsGalleryLoadTemplate = new RibbonButton[_imgLstGalleryTemplates.Images.Count];
         Array commandIds = Enum.GetValues(typeof(LoadTemplateButtonCommand));

         for (uint i = 0; i < _btnsGalleryLoadTemplate.Length; ++i)
         {
            _btnsGalleryLoadTemplate[i] = new RibbonButton(_ribbonMain, (uint) commandIds.GetValue(i))
            {
               Label = LoadTemplateButtonLabelText[(int) i],
               LargeImage = _ribbonMain.ConvertToUIImage((Bitmap) _imgLstGalleryTemplates.Images[(int) i])
            };
            _btnsGalleryLoadTemplate[i].ExecuteEvent += BtnsGalleryLoadTemplate_ExecuteEvent;
         }

         // Add button commands

         IUICollection itemsSource = _drpDnGlryLoadTemplate.ItemsSource;
         itemsSource.Clear();

         foreach (object n in commandIds)
         {
            itemsSource.Add(new GalleryCommandPropertySet()
            {
               CommandID = (uint) n,
               CommandType = CommandType.Action,
            });
         }
      } // DrpDnGlryLoadTemplate_ItemsSourceReady

      void BtnsGalleryLoadTemplate_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         RibbonButton btn = sender as RibbonButton;
         if (btn == null) return;

         var cmd = (LoadTemplateButtonCommand) btn.CommandID;

         byte[] tmpl = null;

         switch (cmd)
         {
            case LoadTemplateButtonCommand.cmdBtnLoadTemplate_Invoice:
               tmpl = GetBinaryResourceByName("TX_Text_Control_Words.Templates.invoice.docx");
               break;

            case LoadTemplateButtonCommand.cmdBtnLoadTemplate_Quote:
               tmpl = GetBinaryResourceByName("TX_Text_Control_Words.Templates.quote.docx");
               break;

            case LoadTemplateButtonCommand.cmdBtnLoadTemplate_Letter:
               tmpl = GetBinaryResourceByName("TX_Text_Control_Words.Templates.letter.docx");
               break;
         }

         LoadTemplateData(tmpl);
      } // BtnsGalleryLoadTemplate_ExecuteEvent

      private void LoadTemplateData(byte[] tmpl)
      {
         if ((tmpl != null)
            && (!_fileHandler.DocumentDirty
            || (_fileHandler.DocumentDirty && FileSaveQuestion())))
         {
            _textControl.Load(
               tmpl,
               TXTextControl.BinaryStreamType.WordprocessingML,
               new LoadSettings { ApplicationFieldFormat = ApplicationFieldFormat.MSWord }
            );
            SetDefaultFieldProperties();
            _fileHandler.DocumentFileName = "";
            _fileHandler.DocumentDirty = false;
         }
      } // LoadTemplateData

      #endregion // Ribbon event handlers

      #region "  Helpers  "

      private void InitRecentItems()
      {
         _recentItemsList = new List<RecentItemsPropertySet>();
         _ribbonRecentItems.RecentItems = _recentItemsList;
      }

      private void CreateRibbonItems_ApplicationMenu()
      {
         _ribbonRecentItems = new RibbonRecentItems(_ribbonMain, (uint)RibbonMarkupCommand_ApplicationMenu.cmdMRUItems);
         _btnExit = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_ApplicationMenu.cmdBtnExit);
         _spltBtnGlrySaveAs = new RibbonSplitButtonGallery(_ribbonMain, (uint)RibbonMarkupCommand_ApplicationMenu.cmdGlrySaveAs);
         _spltBtnGlryPrint = new RibbonSplitButtonGallery(_ribbonMain, (uint)RibbonMarkupCommand_ApplicationMenu.cmdGlryPrint);
         _btnNew = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_ApplicationMenu.cmdBtnNew);
         _btnOpen = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_ApplicationMenu.cmdBtnOpen);
         _btnSave = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_ApplicationMenu.cmdBtnSave);
         _btnOptions = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_ApplicationMenu.cmdBtnOptions);
         _btnPrint = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_ApplicationMenu.cmdBtnPrint);
         _btnUndo = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_ApplicationMenu.cmdBtnUndo);
         _btnRedo = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_ApplicationMenu.cmdBtnRedo);
         _drpDnGlryLoadTemplate = new RibbonDropDownGallery(_ribbonMain, (uint) RibbonMarkupCommand_ApplicationMenu.cmdGlryLoadTemplate);
      }

      private void AttachRibbonEvents_ApplicationMenu()
      {
         _ribbonRecentItems.ExecuteEvent += RecentItems_ExecuteEvent;
         _btnExit.ExecuteEvent += BtnExit_ExecuteEvent;
         _spltBtnGlrySaveAs.ExecuteEvent += SplitBtnGallerySaveAs_ExecuteEvent;
         _spltBtnGlrySaveAs.ItemsSourceReady += BtnGlrySaveAs_ItemsSourceReady;
         _spltBtnGlryPrint.ItemsSourceReady += SplitBtnGalleryPrint_ItemsSourceReady;
         _btnNew.ExecuteEvent += BtnNew_ExecuteEvent;
         _btnOpen.ExecuteEvent += BtnOpen_ExecuteEvent;
         _btnSave.ExecuteEvent += BtnSave_ExecuteEvent;
         _btnOptions.ExecuteEvent += BtnOptions_ExecuteEvent;
         _btnPrint.ExecuteEvent += BtnPrint_ExecuteEvent;
         _btnUndo.ExecuteEvent += BtnUndo_ExecuteEvent;
         _btnRedo.ExecuteEvent += BtnRedo_ExecuteEvent;
         _drpDnGlryLoadTemplate.ItemsSourceReady += DrpDnGlryLoadTemplate_ItemsSourceReady;
      }

      internal void UpdateSaveStatus()
      {
         _btnSave.Enabled = _fileHandler.DocumentDirty;
      }

      #endregion // Helpers
   }
}
