﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				CommonHelpers.cs
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using TXTextControl;
using System.Reflection;
using System.IO;
using System.Windows.Forms;


namespace TX_Text_Control_Words
{
   public partial class FormMain : Form
   {
      #region "  Page numbers  "

      /// <summary>
      /// Inserts a page number field into the current section.
      /// </summary>
      /// <param name="headerFooterType">Only header or footer are allowed.</param>
      private void InsertPageNumber(HeaderFooterType headerFooterType)
      {
         HeaderFooter hdrFtr = null;

         try
         {
            // Get current section
            var sect = _textControl.Sections.GetItem();
            if (sect == null) return;

            switch (headerFooterType)
            {
               case HeaderFooterType.Header:
                  // Create header if none exists
                  if (!HeaderExists()) sect.HeadersAndFooters.Add(headerFooterType);

                  // Use first page header if one exists and input position is on first page of section
                  if ((sect.Start == _textControl.InputPosition.Page)
                     && (sect.HeadersAndFooters.GetItem(HeaderFooterType.FirstPageHeader) != null))
                  {
                     headerFooterType = HeaderFooterType.FirstPageHeader;
                  }

                  hdrFtr = sect.HeadersAndFooters.GetItem(headerFooterType);
                  break;

               case HeaderFooterType.Footer:
                  // Create footer if none exists
                  if (!FooterExists()) sect.HeadersAndFooters.Add(headerFooterType);

                  // Use first page header if one exists and input position is on first page of section
                  if ((sect.Start == _textControl.InputPosition.Page)
                     && (sect.HeadersAndFooters.GetItem(HeaderFooterType.FirstPageFooter) != null))
                  {
                     headerFooterType = HeaderFooterType.FirstPageFooter;
                  }

                  hdrFtr = sect.HeadersAndFooters.GetItem(headerFooterType);
                  break;
            }

            if (hdrFtr == null) return;

            // If the header / footer already contains a page number, break
            if (hdrFtr.PageNumberFields.Count > 0) return;

            var pgNumFld = new PageNumberField
            {
               StartNumber = 1,
               Editable = false,
               DoubledInputPosition = true,
               ShowActivated = true,
            };
            hdrFtr.PageNumberFields.Add(pgNumFld);

         }
         catch (Exception ex)
         {
            // Display information if feature not enabled in current version
            MessageBox.Show(ex.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
            return;
         }
      }

      /// <summary>
      /// Removes all page number fields from all headers and footers of the current section.
      /// </summary>
      private void RemovePageNumbers()
      {
         Section sectCur = null;

         try
         {
            sectCur = _textControl.Sections.GetItem();
            if (sectCur == null) return;

            foreach (HeaderFooter hdrFtr in sectCur.HeadersAndFooters)
            {
               hdrFtr.PageNumberFields.Clear(false);
            }
         }
         catch (Exception ex)
         {
            // Display information if feature not enabled in current version
            MessageBox.Show(ex.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
         }
      }

      #endregion // Page Numbers


      #region "  Headers / Footers  "

      private bool HeaderExists()
      {
         if (_textControl == null) return false;

         try
         {
            return ((_textControl.Sections.GetItem().HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Header) != null)
                        || (_textControl.Sections.GetItem().HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageHeader) != null));
         }
         catch { }

         return false;
      }

      private bool FooterExists()
      {
         if (_textControl == null) return false;

         try
         {
            return ((_textControl.Sections.GetItem().HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Footer) != null)
                        || (_textControl.Sections.GetItem().HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageFooter) != null));
         }
         catch { }

         return false;
      }

      #endregion // Headers / Footers


      #region "  Embedded Resources  "

      internal static string GetResourceByName(string resource)
      {
         var assembly = Assembly.GetExecutingAssembly();
         var reader = new StreamReader(assembly.GetManifestResourceStream(resource));
         return reader.ReadToEnd();
      }

      internal static byte[] GetBinaryResourceByName(string resource)
      {
         var buffer = new byte[4096];  // Data buffer for reading chunks from the binary stream
         var msResult = new MemoryStream();
         BinaryReader reader = null;
         Assembly asm = null;

         try
         {
            asm = Assembly.GetExecutingAssembly();
            reader = new BinaryReader(asm.GetManifestResourceStream(resource));
         }
         catch (Exception ex)
         {
            MessageBox.Show(ex.Message, "Whoops", MessageBoxButtons.OK, MessageBoxIcon.Error);
            return null;
         }

         for (; ; )
         {
            int bytesRead = reader.Read(buffer, 0, buffer.Length);
            if (bytesRead <= 0) break;
            msResult.Write(buffer, 0, bytesRead);
         }

         return msResult.ToArray();
      }

      #endregion // Embedded Resources
   }
}
