﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				DbConnectionAdapter.cs
** description:	This file contains an adapter class wrapping a DbConnection object.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Diagnostics;
using System.Reflection;
using System.Windows.Forms;


namespace TX_Text_Control_Words
{
   /// <summary>
   /// Adapter class for DbConnection objects.
   /// </summary>
   internal class DataProviderConnectionAdapter : DbConnectionAdapter
   {
      private DbConnection _dbCon;              // DB connection
      private DbProviderFactory _dbProvFact;    // DB provider factory needed to create DB commands
      private DataSet _dsRelsAndTbls;           // DataSet holding tables and data relations which were created
                                                // by the user while a database connection was open

      internal override List<DataRelation> DataRelations
      {
         get
         {
            var result = new List<DataRelation>();

            if (_dsRelsAndTbls != null)
            {
               foreach (DataRelation rel in _dsRelsAndTbls.Relations)
               {
                  result.Add(rel);
               }
            }

            return result;
         }
      }

      // SQL command string
      // {0}: table name
      private const string QueryStringTemplate = "select * from [{0}]";

      internal DataProviderConnectionAdapter(string strDataProviderName, string strConnectionString)
      {
         Connect(strDataProviderName, strConnectionString);
         GetTableAndViewNames();
      }

      internal override DataTable GetTable(string strTable)
      {
         if (_dbCon == null) return null;

         // If table is already part of data relations return it
         if ((_dsRelsAndTbls != null) && _dsRelsAndTbls.Tables.Contains(strTable))
         {
            return _dsRelsAndTbls.Tables[strTable];
         }

         // Fetch the complete table:

         var table = new DataTable(strTable);
         var adapter = _dbProvFact.CreateDataAdapter();
         var command = _dbProvFact.CreateCommand();

         command.Connection = _dbCon;
         command.CommandType = System.Data.CommandType.Text;
         command.CommandText = GetQueryString(strTable);

         try
         {
            adapter.SelectCommand = command;
            adapter.Fill(table);
         }
         catch (Exception exc)
         {
            MessageBox.Show(exc.Message, "Table Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            table = null;
         }

         // Store table
         if (table != null)
         {
            if (_dsRelsAndTbls == null) _dsRelsAndTbls = new DataSet();
            _dsRelsAndTbls.Tables.Add(table);
         }

         return table;
      }

      internal override void SelectTable(string strTableName)
      {
         if (_dbCon == null) return;
         _selectedTable = new DataTableAdapter(GetTable(strTableName));
      }

      internal override void Close()
      {
         if (_dbCon == null) return;
         if (_dbCon.State != ConnectionState.Closed) _dbCon.Close();
         _dbCon = null;
         if (_dsRelsAndTbls != null) _dsRelsAndTbls.Clear();
         _dsRelsAndTbls = null;
      }

      internal override void GetTableAndViewNames()
      {
         _tableNames = new List<string>();
         _viewNames = new List<string>();

         if (_dbCon == null) return;

         try
         {
            // Get table names from database connection
            var tables = _dbCon.GetSchema("Tables");
            foreach (DataRow table in tables.Rows)
            {
               string strTblName = table["TABLE_NAME"].ToString();
               if (strTblName.ToLower().StartsWith("msys")) continue;   // Leave out ms access system tables
               TableNames.Add(strTblName);
            }

            // Get view names from database connection
            var views = _dbCon.GetSchema("Views");
            foreach (DataRow view in views.Rows)
            {
               ViewNames.Add(view["TABLE_NAME"].ToString());
            }
         }
         catch (Exception exc)
         {
            MessageBox.Show(exc.Message, "Database connection", MessageBoxButtons.OK, MessageBoxIcon.Error);
            if ((_dbCon != null) && (_dbCon.State != ConnectionState.Closed)) _dbCon.Close();
            _dbCon = null;
            _dbProvFact = null;
            if (_dsRelsAndTbls != null) _dsRelsAndTbls.Clear();
            _dsRelsAndTbls = null;
         }
      }

      private void Connect(string strDataProviderName, string strConnectionString)
      {
         if ((_dbCon != null) && (_dbCon.State != ConnectionState.Closed)) _dbCon.Close();
         _dbCon = null;
         _dbProvFact = null;
         _selectedTable = null;
         if (_dsRelsAndTbls != null) _dsRelsAndTbls.Clear();
         _dsRelsAndTbls = null;

         try
         {
            _dbProvFact = DbProviderFactories.GetFactory(strDataProviderName);
            _dbCon = _dbProvFact.CreateConnection();
            _dbCon.ConnectionString = strConnectionString;
            _dbCon.Open();

            _dsRelsAndTbls = new DataSet();
         }
         catch (Exception exc)
         {
            MessageBox.Show(exc.Message, "Database connection", MessageBoxButtons.OK, MessageBoxIcon.Error);
            if ((_dbCon != null) && (_dbCon.State != ConnectionState.Closed)) _dbCon.Close();
            _dbCon = null;
            _dbProvFact = null;
            if (_dsRelsAndTbls != null) _dsRelsAndTbls.Clear();
            _dsRelsAndTbls = null;
            return;
         }
      }

      /// <summary>
      /// Returns a list of tables from the database.
      /// </summary>
      /// <param name="tableNames">Table names.</param>
      /// <returns>Tables.</returns>
      internal override DataSet GetDataSet(List<string> tableNames)
      {
         if (_dbCon == null) return null;

         var ds = new DataSet();
         var dbDataAdap = _dbProvFact.CreateDataAdapter();
         var dbCmd = _dbProvFact.CreateCommand();

         dbCmd.Connection = _dbCon;
         dbCmd.CommandType = CommandType.Text;

         foreach (string strName in tableNames)
         {
            dbCmd.CommandText = GetQueryString(strName);

            try
            {
               dbDataAdap.SelectCommand = dbCmd;
               dbDataAdap.Fill(ds, strName);
            }
            catch (Exception exc)
            {
               Debug.WriteLine("Problem in method " + MethodInfo.GetCurrentMethod().Name + ": " + exc.Message);
            }
         }

         return ds;
      }

      /// <summary>
      /// Generates a general query string returning all rows of a table.
      /// </summary>
      /// <param name="strTableName">Table name.</param>
      /// <returns>Query string.</returns>
      private static string GetQueryString(string strTableName)
      {
         return string.Format(QueryStringTemplate, strTableName);
      }

      /// <summary>
      /// Adds a data relation to the currently selected database and table.
      /// </summary>
      /// <param name="mainTableName">Main table name.</param>
      /// <param name="mainColName">Main table column name.</param>
      /// <param name="childTableName">Child table name.</param>
      /// <param name="childColName">Child table column name.</param>
      internal override bool AddDataRelation(string mainTableName, string mainColName, string childTableName, string childColName)
      {
         // Create new data set if not present by now
         if (_dsRelsAndTbls == null) _dsRelsAndTbls = new DataSet();

         // Fetch tables if necessary
         if (!_dsRelsAndTbls.Tables.Contains(mainTableName))
         {
            _dsRelsAndTbls.Tables.Add(GetTable(mainTableName));
         }
         if (!_dsRelsAndTbls.Tables.Contains(childTableName))
         {
            _dsRelsAndTbls.Tables.Add(GetTable(childTableName));
         }

         // Create and add relation
         try
         {
            var relation = new DataRelation(
               Guid.NewGuid().ToString(),
               _dsRelsAndTbls.Tables[mainTableName].Columns[mainColName],
               _dsRelsAndTbls.Tables[childTableName].Columns[childColName]
            );
            _dsRelsAndTbls.Relations.Add(relation);
         }
         catch (Exception exc)
         {
            MessageBox.Show("Data relation could not be created: " + exc.Message, "Data Relation", MessageBoxButtons.OK, MessageBoxIcon.Warning);
            return false;
         }

         return true;
      }
   } // class DataProviderConnectionAdapter
}
