﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				FormMain.cs
** description:	This file contains the main form.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using RibbonLib;
using RibbonLib.Controls;
using RibbonLib.Controls.Events;
using RibbonLib.Interop;
using System.Diagnostics;
using System.Reflection;
using Keyoti.RapidSpell;


namespace TX_Text_Control_Words
{
   /// <summary>
   /// The main form.
   /// </summary>
   public partial class FormMain : Form
   {
      #region "  Private fields  "

      private FileHandler _fileHandler;
      private FileDragDropHandler _fileDragDrop;
      private bool _bLoadFileOnCreate = false;
      private RibbonHelpButton _btnHelp;
      private RibbonTabGroup _tabGroupTableTools;
      private RibbonTabGroup _tabGroupObjectTools;
      private TXTextControl.FrameBase _objSel;              // Currently selected object
      private TXTextControl.FrameBase _objSelPrev;          // Previously selected object (Text Control deselection workaround)


      private const string ExpressEditionInfoMessage = "Not available in Express Edition.";
      private const string MsgDocDoesNotContainAny = "The document doesn't contain any {0}.";
      private const uint HelpButtonCommandID = 1999;
      private const uint DynamicCommandIdStart = 10000;

      #endregion // Private fields
      RapidSpellDialog rapidSpellDialog1;
      #region "  Constructors  "

      /// <summary>
      /// Creates a main form and opens a document given as the first string argument.
      /// </summary>
      /// <param name="args">List of string arguments. The first argument is considered 
      /// a document file name to be opened at startup.</param>
      public FormMain(string[] args)
      {
         InitializeComponent();

         _textControl.RulerBar = _rulerBarHor;
         _textControl.StatusBar = _statusBar;
         _textControl.VerticalRulerBar = _rulerBarVert;

         _fileHandler = new FileHandler(this, _textControl);
         _fileDragDrop = new FileDragDropHandler();

         if (args.Length > 0)
         {
            _fileHandler.DocumentFileName = args[0];
            _bLoadFileOnCreate = true;
         }

         rapidSpellDialog1 = new RapidSpellDialog();

         // rapidSpellDialog1.ContextMenuStripDefault = null;
         rapidSpellDialog1.AddTextComponent(_textControl);
         rapidSpellDialog1.ShowSuggestionsContextMenu = false;
      }

      #endregion // Constructors

      #region "  Form event handlers  "

      private void FormMain_Load(object sender, EventArgs e)
      {
         Text = ProductName;

         // Create ribbon element objects and attach events
         CreateRibbonItems();
         AttachRibbonEvents();
         InitRecentItems();
         InitInputFormatDependentControls();

         LoadAppSettings();

         AttachTextControlEvents();

         // Init list of formatting styles shown in the styles gallery
         TextControl_FormattingStyleListChanged(this, null);

         _fileHandler.RecentFiles = _recentItemsList;
         _fileHandler.InitRecentFiles();

         UpdateZoomCheckBoxes();

         _tglBtnTblViewGridlines.BooleanValue = _textControl.Tables.GridLines;
         _tglBtnShowTextFrameMarkerLines.BooleanValue = _textControl.TextFrameMarkerLines;
         _tglBtnHighlightMergeFields.BooleanValue = _bHighlightFields;
         _tglBtnShowBookmarkMarkers.BooleanValue = _textControl.DocumentTargetMarkers;

         SelectCurrentStyleItem();


         ToggleFieldDisplayButtons();

         _dlgMergeWait.Owner = this;

         if (_bLoadFileOnCreate)
         {
            if (!_fileHandler.FileOpen()) return;
            if (_fileHandler.DocumentFileName != "")
            {
               this.Text = _fileHandler.PrintDocumentName;
            }
         }

         _fileHandler.DocumentDirty = true;

         UpdateGUIState();
      }

      private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
      {
         if (_fileHandler.DocumentDirty)
         {
            e.Cancel = !FileSaveQuestion();
         }
         SaveAppSettings();
      }

      #endregion // Form event handlers

      #region "  Ribbon event handlers  "

      void BtnHelp_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         About();
      }

      #endregion // Ribbon event handlers

      #region "  Helpers  "

      private void CreateRibbonItems()
      {
         _btnHelp = new RibbonHelpButton(_ribbonMain, HelpButtonCommandID);
         _tabGroupTableTools = new RibbonTabGroup(_ribbonMain, (uint) RibbonMarkupCommand_TabGrpTableTools.cmdTabGrpTableTools);
         _tabGroupObjectTools = new RibbonTabGroup(_ribbonMain, (uint) RibbonMarkupCommand_TabGrpObjectTools.cmdTabGrpObjectTools);

         CreateRibbonItems_TabHome();
         CreateRibbonItems_TabInsert();
         CreateRibbonItems_TabPageLayout();
         CreateRibbonItems_TabMailings();
         CreateRibbonItems_TabView();
         CreateRibbonItems_ApplicationMenu();
         CreateRibbonItems_TabGrpTableTools();
         CreateRibbonItems_TabGrpObjectTools();

      }

      private void AttachRibbonEvents()
      {
         _btnHelp.ExecuteEvent += BtnHelp_ExecuteEvent;

         AttachRibbonEvents_TabHome();
         AttachRibbonEvents_TabInsert();
         AttachRibbonEvents_TabPageLayout();
         AttachRibbonEvents_TabMailings();
         AttachRibbonEvents_TabView();
         AttachRibbonEvents_ApplicationMenu();
         AttachRibbonEvents_TabGrpTableTools();
         AttachRibbonEvents_TabGrpObjectTools();

      }

      private void UpdateGUIState()
      {
         UpdateCopyPaste();
         UpdateUndoRedo();
         UpdateViewCheckBoxes();
         UpdateLinkButtons();

         UpdateContextualTableTabGroup();

         _btnRemoveHeader.Enabled = HeaderExists();
         _btnRemoveFooter.Enabled = FooterExists();
      }

      private void UpdateLinkButtons()
      {
         // Hypertext links are not available in the Standard version. Accessing them  
         // would throw an exception if this sample program is used with a Standard version
         // of Text Control.
         try
         {
            _btnEditHyperlink.Enabled = (_textControl.HypertextLinks.GetItem() != null) || (_textControl.DocumentLinks.GetItem() != null);
            _btnEditBookmark.Enabled = (_textControl.DocumentTargets.GetItem() != null);
         }
         catch
         { }
      }

      private void UntoggleToggleButtons(RibbonToggleButton[] btns)
      {
         UntoggleToggleButtons(btns, (RibbonToggleButton) null);
      }

      void UntoggleToggleButtons(RibbonToggleButton[] buttons, RibbonToggleButton exceptThisOne)
      {
         UntoggleToggleButtons(buttons, new List<RibbonToggleButton>(new RibbonToggleButton[] { exceptThisOne }));
      }

      void UntoggleToggleButtons(RibbonToggleButton[] buttons, List<RibbonToggleButton> exceptThese)
      {
         foreach (RibbonToggleButton btn in buttons)
         {
            if (exceptThese.Contains(btn)) continue;
            btn.BooleanValue = false;
         }
      }

      private void UntoggleCheckboxes(RibbonCheckBox[] chk)
      {
         UntoggleCheckboxes(chk, (RibbonCheckBox) null);
      }

      private void UntoggleCheckboxes(RibbonCheckBox[] checkBoxes, RibbonCheckBox exceptThisOne)
      {
         UntoggleCheckboxes(checkBoxes, new List<RibbonCheckBox>(new RibbonCheckBox[] { exceptThisOne }));
      }

      void UntoggleCheckboxes(RibbonCheckBox[] checkBoxes, List<RibbonCheckBox> exceptThese)
      {
         foreach (RibbonCheckBox cb in checkBoxes)
         {
            if (exceptThese.Contains(cb)) continue;
            cb.BooleanValue = false;
         }
      }

      private void LoadAppSettings()
      {
         // Take over initial resizing
         this.StartPosition = FormStartPosition.Manual;

         // Resize form
         this.Size = Properties.Settings.Default.LastWindowSize;
         this.Location = Properties.Settings.Default.LastWindowPos;
         this.WindowState = Properties.Settings.Default.LastWindowState;
      }

      private void SaveAppSettings()
      {
         if (this.WindowState == FormWindowState.Normal)
         {
            Properties.Settings.Default.LastWindowPos = new System.Drawing.Point((int) this.Left, (int) this.Top);
            Properties.Settings.Default.LastWindowSize = new System.Drawing.Size((int) this.Width, (int) this.Height);
         }
         else
         {
            Properties.Settings.Default.LastWindowPos
               = new System.Drawing.Point((int) this.RestoreBounds.Left, (int) this.RestoreBounds.Top);
            Properties.Settings.Default.LastWindowSize
               = new System.Drawing.Size((int) this.RestoreBounds.Width, (int) this.RestoreBounds.Height);
         }
         Properties.Settings.Default.Save();
      }

      /// <summary>
      /// Attaches text control events which can't be set in the properies explorer.
      /// </summary>
      private void AttachTextControlEvents()
      {
         // Font properties:

         _textControl.InputFormat.BoldChanged += InputFormat_BoldChanged;
         _textControl.InputFormat.FontFamilyChanged += InputFormat_FontFamilyChanged;
         _textControl.InputFormat.FontSizeChanged += InputFormat_FontSizeChanged;
         _textControl.InputFormat.ItalicChanged += InputFormat_ItalicChanged;
         _textControl.InputFormat.StrikeoutChanged += InputFormat_StrikeoutChanged;
         _textControl.InputFormat.SubscriptChanged += InputFormat_SubscriptChanged;
         _textControl.InputFormat.SuperscriptChanged += InputFormat_SuperscriptChanged;
         _textControl.InputFormat.TextBackColorChanged += InputFormat_TextBackColorChanged;
         _textControl.InputFormat.TextColorChanged += InputFormat_TextColorChanged;
         _textControl.InputFormat.UnderlineChanged += InputFormat_UnderlineChanged;

         // Paragraph properties:

         _textControl.InputFormat.AllFrameLinesChanged += InputFormat_AllFrameLinesChanged;
         _textControl.InputFormat.BottomFrameLineChanged += InputFormat_BottomFrameLineChanged;
         _textControl.InputFormat.BoxFrameChanged += InputFormat_BoxFrameChanged;
         _textControl.InputFormat.BulletCharacterChanged += InputFormat_BulletCharacterChanged;
         _textControl.InputFormat.BulletedListChanged += InputFormat_BulletedListChanged;
         _textControl.InputFormat.CenteredChanged += InputFormat_CenteredChanged;
         _textControl.InputFormat.InnerHorizontalFrameLinesChanged += InputFormat_InnerHorizontalFrameLinesChanged;
         _textControl.InputFormat.InnerVerticalFrameLinesChanged += InputFormat_InnerVerticalFrameLinesChanged;
         _textControl.InputFormat.JustifiedChanged += InputFormat_JustifiedChanged;
         _textControl.InputFormat.LeftAlignedChanged += InputFormat_LeftAlignedChanged;
         _textControl.InputFormat.LeftFrameLineChanged += InputFormat_LeftFrameLineChanged;
         _textControl.InputFormat.LineSpacingChanged += InputFormat_LineSpacingChanged;
         _textControl.InputFormat.NumberedListChanged += InputFormat_NumberedListChanged;
         _textControl.InputFormat.NumberedListFormatChanged += InputFormat_NumberedListFormatChanged;
         _textControl.InputFormat.RightAlignedChanged += InputFormat_RightAlignedChanged;
         _textControl.InputFormat.RightFrameLineChanged += InputFormat_RightFrameLineChanged;
         _textControl.InputFormat.StructuredListChanged += InputFormat_StructuredListChanged;
         _textControl.InputFormat.StructuredListFormatChanged += InputFormat_StructuredListFormatChanged;
         _textControl.InputFormat.TopFrameLineChanged += InputFormat_TopFrameLineChanged;

         // Style properties:

         _textControl.InputFormat.StyleNameChanged += InputFormat_StyleNameChanged;

         // Various:

         _textControl.GotFocus += TextControl_GotFocus;
      }

      private void UpdateCurrentObject()
      {
         GetSelectedObject();
         UpdateContextualTabs();
      }

      private void UpdateContextualTabs()
      {
         UpdateContextualObjectTabGroup();
         UpdateContextualTableTabGroup();

      }

      public void About()
      {
         AboutBox.Show();
      }

      private void FormattingStyleListChanged()
      {
         IUICollection itemsSource = _spltBtnGlryStyles.ItemsSource;
         itemsSource.Clear();

         foreach (string styleName in _textControl.InputFormat.StyleNames)
         {
            itemsSource.Add(new GalleryItemPropertySet()
            {
               Label = styleName,
               CategoryID = 1,
            });
         }
      }

      #endregion // Helpers
   }
}
