﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				Mailings.cs
** description:	This file contains the implementation of the mail merge functions.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Windows.Forms;
using TXTextControl.DocumentServer.Fields;


namespace TX_Text_Control_Words
{
   public partial class FormMain : Form
   {
      private FieldDisplayMode _fldDispModeCur;
      private bool _bHighlightFields = true;

      private const string MsgNoFields = "The document does not contain any fields.";

      private enum FieldDisplayMode
      {
         ShowFieldText,
         ShowFieldCodes,
      }

      /// <summary>
      /// Sets document wide default field properties.
      /// </summary>
      internal void SetDefaultFieldProperties()
      {
         try
         {
            foreach (TXTextControl.IFormattedText textPart in _textControl.TextParts)
            {
               foreach (TXTextControl.ApplicationField fld in textPart.ApplicationFields)
               {
                  fld.Editable = false;
                  fld.DoubledInputPosition = true;
                  fld.ShowActivated = _bHighlightFields;
               }
            }
         }
         catch { }
      }

      private MergeField InsertMergeField(string strName)
      {
         MergeField mf = CreateMergeField(strName);
         _textControl.ApplicationFields.Add(mf.ApplicationField);
         return mf;
      }

      private MergeField CreateMergeField(string strName)
      {
         var mf = new MergeField();

         mf.ApplicationField.DoubledInputPosition = true;
         mf.ApplicationField.Editable = false;
         mf.ApplicationField.ShowActivated = _bHighlightFields;

         if (strName != string.Empty)
         {
            mf.Name = strName;

            switch (_fldDispModeCur)
            {
               case FieldDisplayMode.ShowFieldCodes:
                  string strSwitches = "";
                  foreach (var fs in mf.ApplicationField.Parameters) strSwitches += " " + fs;
                  mf.Text = "{" + mf.TypeName + strSwitches + " }";
                  break;

               case FieldDisplayMode.ShowFieldText:
                  mf.Text = "«" + strName + "»";
                  break;
            }
         }

         return mf;
      }

      private void InsertMergeField()
      {
         var newMergeField = CreateMergeField("");
         if (newMergeField.ShowDialog() == TXTextControl.DocumentServer.Fields.DialogResult.OK)
         {
            newMergeField.Text = "«" + newMergeField.Name + "»";
            _textControl.ApplicationFields.Add(newMergeField.ApplicationField);
         }
      }

      private void InsertIfField()
      {
         var newIfField = new IfField();
         newIfField.ApplicationField.DoubledInputPosition = true;
         newIfField.ApplicationField.Editable = false;
         newIfField.ApplicationField.ShowActivated = _bHighlightFields;

         if (newIfField.ShowDialog() == TXTextControl.DocumentServer.Fields.DialogResult.OK)
         {
            _textControl.ApplicationFields.Add(newIfField.ApplicationField);
         }
      }

      private void InsertDateField()
      {
         var newDateField = new DateField() { Format = "dd.MM.yyyy" };
         newDateField.ApplicationField.DoubledInputPosition = true;
         newDateField.ApplicationField.Editable = false;
         newDateField.ApplicationField.ShowActivated = _bHighlightFields;

         if (newDateField.ShowDialog() == TXTextControl.DocumentServer.Fields.DialogResult.OK)
         {
            _textControl.ApplicationFields.Add(newDateField.ApplicationField);
         }
      }

      private void InsertIncludeTextField()
      {
         var newIncludeTextField = new IncludeText();
         newIncludeTextField.ApplicationField.DoubledInputPosition = true;
         newIncludeTextField.ApplicationField.Editable = false;
         newIncludeTextField.ApplicationField.ShowActivated = _bHighlightFields;

         if (newIncludeTextField.ShowDialog() == TXTextControl.DocumentServer.Fields.DialogResult.OK)
         {
            _textControl.ApplicationFields.Add(newIncludeTextField.ApplicationField);
         }
      }

      private void DeleteField()
      {
         if (!FieldAtCurrentPos())
         {
            MessageBox.Show(
               "There is no field at the current input position.", 
               ProductName, 
               MessageBoxButtons.OK, 
               MessageBoxIcon.Information
            );
            return;
         }

         try
         {
            var field = _textControl.ApplicationFields.GetItem();
            _textControl.ApplicationFields.Remove(field);
         }
         catch (Exception ex)
         {
            MessageBox.Show(ex.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
         }
      }

      private bool FieldAtCurrentPos()
      {
         try
         {
            var field = _textControl.ApplicationFields.GetItem();
            return field != null;
         }
         catch { return false; }
      }

      private void FieldSettings()
      {
         try
         {
            var field = _textControl.ApplicationFields.GetItem();
            if (field == null)
            {
               MessageBox.Show(
                  "There is no field at the current input position.", 
                  ProductName, 
                  MessageBoxButtons.OK, 
                  MessageBoxIcon.Information
               );
               return;
            }

            switch (field.TypeName)
            {
               case MergeField.TYPE_NAME:
                  var mergeField = new MergeField(field);
                  mergeField.ShowDialog();
                  break;

               case DateField.TYPE_NAME:
                  var dateField = new DateField(field);
                  dateField.ShowDialog();
                  break;

               case IncludeText.TYPE_NAME:
                  var includeTextField = new IncludeText(field);
                  includeTextField.ShowDialog();
                  break;

               case IfField.TYPE_NAME:
                  var ifField = new IfField(field);
                  ifField.ShowDialog();
                  break;
            }

            // Update field values after editing field
            UpdateFieldValues();
         }
         catch { }
      }

      private void UpdateFieldValues()
      {
         try
         {
            switch (_fldDispModeCur)
            {
               case FieldDisplayMode.ShowFieldCodes:
                  foreach (TXTextControl.IFormattedText textPart in _textControl.TextParts)
                  {
                     foreach (TXTextControl.ApplicationField appField in textPart.ApplicationFields)
                     {
                        string fieldSwitches = String.Empty;

                        foreach (var fieldSwitch in appField.Parameters)
                           fieldSwitches += " " + fieldSwitch;

                        appField.Text = "{ " + appField.TypeName + " " + fieldSwitches + " }";
                     }
                  }

                  break;

               case FieldDisplayMode.ShowFieldText:
                  foreach (TXTextControl.IFormattedText textPart in _textControl.TextParts)
                  {
                     foreach (TXTextControl.ApplicationField appField in textPart.ApplicationFields)
                     {
                        switch (appField.TypeName)
                        {
                           case MergeField.TYPE_NAME:
                              var mergeField = new MergeField(appField);
                              mergeField.Text = "«" + mergeField.Name + "»";
                              break;

                           case IfField.TYPE_NAME:
                              var ifField = new IfField(appField);
                              ifField.Text = "{" + ifField.TypeName + "}";
                              break;

                           case DateField.TYPE_NAME:
                              var dateField = new DateField(appField);
                              dateField.Text = "{" + dateField.TypeName + "}";
                              break;

                           case IncludeText.TYPE_NAME:
                              var includeText = new IncludeText(appField);
                              includeText.ApplicationField.Text = "{" + includeText.TypeName + "}";
                              break;
                        }
                     }
                  }

                  break;
            }
         }
         catch { }
      }

      private bool DocumentContainsFields()
      {
         foreach (TXTextControl.IFormattedText textPart in _textControl.TextParts)
         {
            if ((textPart.ApplicationFields != null) && (textPart.ApplicationFields.Count > 0)) return true;
         }

         return false;
      }

      private string StreamTypeToFileExt(TXTextControl.StreamType streamType)
      {
         switch (streamType)
         {
            case TXTextControl.StreamType.AdobePDF:
            case TXTextControl.StreamType.AdobePDFA:
               return "pdf";

            case TXTextControl.StreamType.CascadingStylesheet:
               return "css";

            case TXTextControl.StreamType.HTMLFormat:
               return "html";

            case TXTextControl.StreamType.InternalFormat:
            case TXTextControl.StreamType.InternalUnicodeFormat:
               return "tx";

            case TXTextControl.StreamType.MSWord:
               return "doc";

            case TXTextControl.StreamType.PlainAnsiText:
            case TXTextControl.StreamType.PlainText:
               return "txt";

            case TXTextControl.StreamType.RichTextFormat:
               return "rtf";

            case TXTextControl.StreamType.WordprocessingML:
               return "docx";

            case TXTextControl.StreamType.XMLFormat:
               return "xml";
         }

         return string.Empty;
      }
   }
}