﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				Mailings.cs
** description:	This file contains the implementation of the mail merge functions only used in the Ribbon
**                based demos.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Printing;
using System.IO;
using System.Windows.Forms;
using RibbonLib;
using TXTextControl.DataVisualization;
using TXTextControl.DocumentServer;
using TXTextControl.DocumentServer.Fields;
using System.Threading;


namespace TX_Text_Control_Words
{
   public partial class FormMain : Form
   {
      public delegate void InvokeDelegate();

      private DbConnectionAdapter _dbCon;
      private byte[] _cachedDoc;    // Cache for current document during preview (Internal unicode format)
      private List<byte[]> _previewDocs = new List<byte[]>();  // List of documents created for preview

      private const string MsgNoDataSource = "No data source selected.";
      private const string MsgNoTableSelected = "No table selected.";

      private void InsertMergeBlock(string strName)
      {
         if ((strName == string.Empty) && (_textControl.Selection.Length == 0))
         {
            MessageBox.Show("No text selected.", ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
            return;
         }

         // Determine block boundaries
         int nStart = _textControl.Selection.Start,
             nEnd = nStart + _textControl.Selection.Length;

         // Validate block boundaries
         if (_textControl.Selection.Length != 0)
         {
            if (!ValidateMergeBlockBoundaries(nStart, nEnd))
            {
               MessageBox.Show("Merge blocks must not overlap.", ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
               return;
            }
         }

         if (strName == string.Empty)
         {
            // Ask user for block name
            FrmInputBox.ShowInputBox("Block name:", ref strName);
            if (strName == string.Empty) return;
         }
         else if (_textControl.Selection.Length == 0)
         {
            // If a block name is given and there is no selection, show dialog
            // to let the user choose what to do (table or paragraph containing 
            // fields dependent on selected table name)
            InsertMergeBlockTemplate(strName);
            return;
         }

         // Prepare block
         var dtStart = new TXTextControl.DocumentTarget(MergeBlock.BlockStartPrefix + strName) { Deleteable = false };
         var dtEnd = new TXTextControl.DocumentTarget(MergeBlock.BlockEndPrefix + strName) { Deleteable = false };

         // Insert block into document
         _textControl.InputPosition = new TXTextControl.InputPosition(nStart);
         _textControl.DocumentTargets.Add(dtStart);
         _textControl.InputPosition = new TXTextControl.InputPosition(nEnd);
         _textControl.DocumentTargets.Add(dtEnd);
      }

      private void DeleteMergeBlock()
      {
         var blocks = GetMergeBlocks();
         if (blocks.Count == 0)
         {
            MessageBox.Show(
               string.Format(MsgDocDoesNotContainAny, "merge blocks"), 
               ProductName, 
               MessageBoxButtons.OK, 
               MessageBoxIcon.Information
            );
            return;
         }

         var dlg = new DeleteMergeBlockDialog(blocks, _textControl);
         dlg.ShowDialog();
      }

      private void ListTablesAndViews()
      {
         var itemsSource = _drpDnGlryMailMrgSelTable.ItemsSource;
         itemsSource.Clear();

         if (_dbCon == null) return;

         // Get table names from database connection
         foreach (var strTblName in _dbCon.TableNames)
         {
            itemsSource.Add(new GalleryItemPropertySet
            {
               Label = strTblName,
               CategoryID = 1,
               ItemImage = _ribbonMain.ConvertToUIImage((Bitmap) _imgLstMailings.Images["tables"]),
            });
         }

         // Get view names from database connection
         foreach (var strViewName in _dbCon.ViewNames)
         {
            itemsSource.Add(new GalleryItemPropertySet
            {
               Label = strViewName,
               CategoryID = 2,
               ItemImage = _ribbonMain.ConvertToUIImage((Bitmap) _imgLstMailings.Images["views"]),
            });
         }
      }

      private int _nMaxRowsPreview;
      private FrmMergeWaitDialog _dlgMergeWait = new FrmMergeWaitDialog();
      private void PreviewMergeResult(int nMaxRows)
      {
         // Store max value
         _nMaxRowsPreview = nMaxRows;

         // Merge
         MergeToMemory(out _previewDocs, _dbCon);

         // Show first data record
         GoToFirstRecord();
      }

      private void PreviewMergeResultAsync(int nMaxRows)
      {
         ThreadPool.QueueUserWorkItem(PreviewMergeResultWaitCallback, nMaxRows);
      }

      private void PreviewMergeResultWaitCallback(object state)
      {
         int nMaxRows = (int) state;

         // Show merge wait dialog modally in UI thread to block 
         // interaction with main form
         BeginInvoke(new InvokeDelegate(ShowMergeWaitDialog));

         PreviewMergeResult(nMaxRows);

         // Close merge wait dialog
         BeginInvoke(new InvokeDelegate(CloseMergeWaitDialog));
      }

      private List<byte[]> _memMergeTmpResult;
      private int _nRecordCur;        // Currently displayed merge result
      private int _nDataRwosMerged;   // Data rows merged to memory so far
      private void MergeToMemory(out List<byte[]> result, DbConnectionAdapter dbCon)
      {
         _memMergeTmpResult = new List<byte[]>();
         _nDataRwosMerged = 0;

         // Save current document to memory
         byte[] template;
         _textControl.Save(out template, TXTextControl.BinaryStreamType.InternalUnicodeFormat);

         try
         {
            // Merge data:

            var mailMerge = new MailMerge() { TextComponent = _textControl };
            mailMerge.LoadTemplateFromMemory(template, FileFormat.InternalUnicodeFormat);

            // Blocks: 
            var mergeBlockTableNames = GetMergeBlockNames();
            if (mergeBlockTableNames.Count > 0)
            {
               mailMerge.MergeBlocks(dbCon.GetDataSet(mergeBlockTableNames));
            }

            // Register event handler which is fired after each merged data row
            mailMerge.DataRowMerged += MailMerge_DataRowMergedToMemory;

            // Start merging
            var tbl = dbCon.SelectedTable == null ? null : dbCon.SelectedTable.DataTable;
            try
            {
               mailMerge.Merge(tbl, false);
            }
            catch (MergeCanceledException) { }

            // If no table was selected, only blocks were merged and the DataRowMerged 
            // event has never been fired. In this case _memMergeTmpResult is empty
            // and the merge result is the content of the mailMerge object
            if (_memMergeTmpResult.Count == 0)
            {
               byte[] doc;
               mailMerge.SaveDocumentToMemory(
                  out doc,
                  TXTextControl.BinaryStreamType.InternalUnicodeFormat,
                  new TXTextControl.SaveSettings()
               );
               _memMergeTmpResult.Add(doc);
            }
         }
         catch (Exception exc)
         {
            MessageBox.Show(
               string.Format("Merge failed: {0}", exc.Message), 
               ProductName, 
               MessageBoxButtons.OK, 
               MessageBoxIcon.Error
            );
         }

         result = _memMergeTmpResult;
         _memMergeTmpResult = null;
         _nRecordCur = 0;
      }

      void MailMerge_DataRowMergedToMemory(object sender, MailMerge.DataRowMergedEventArgs e)
      {
         _memMergeTmpResult.Add(e.MergedRow);
         ++_nDataRwosMerged;

         // Cancel merge process if max number of rows has been merged
         if (_nDataRwosMerged == _nMaxRowsPreview) e.Cancel = true;
      }

      private void BufferCurrentDoc()
      {
         _textControl.Save(out _cachedDoc, TXTextControl.BinaryStreamType.InternalUnicodeFormat);
      }

      private void ReloadBufferedDoc()
      {
         if (_cachedDoc == null) return;

         _previewDocs = null;

         _textControl.Load(
            _cachedDoc,
            TXTextControl.BinaryStreamType.InternalUnicodeFormat,
            new TXTextControl.LoadSettings { ApplicationFieldFormat = TXTextControl.ApplicationFieldFormat.MSWord }
         );

         _cachedDoc = null;
      }

      private void GoToFirstRecord()
      {
         if ((_previewDocs == null) || (_previewDocs.Count == 0)) return;

         _nRecordCur = 0;
         DisplayCurrentRecord();
      }

      private void GoToPreviousRecord()
      {
         if ((_previewDocs == null) || (_previewDocs.Count == 0)) return;

         _nRecordCur = Math.Max(0, --_nRecordCur);
         DisplayCurrentRecord();
      }

      private void GoToNextRecord()
      {
         if ((_previewDocs == null) || (_previewDocs.Count == 0)) return;

         _nRecordCur = Math.Min(_previewDocs.Count - 1, ++_nRecordCur);
         DisplayCurrentRecord();
      }

      private void GoToLastRecord()
      {
         if ((_previewDocs == null) || (_previewDocs.Count == 0)) return;

         _nRecordCur = _previewDocs.Count - 1;
         DisplayCurrentRecord();
      }

      private void DisplayCurrentRecord()
      {
         if (_previewDocs.Count == 0) return;

         _textControl.Load(
            _previewDocs[_nRecordCur],
            TXTextControl.BinaryStreamType.InternalUnicodeFormat,
            new TXTextControl.LoadSettings { ApplicationFieldFormat = TXTextControl.ApplicationFieldFormat.MSWord }
         );
      }
      /// <summary>
      /// Returns names of all merge blocks in current document.
      /// </summary>
      /// <returns>List of merge block names in the current document.</returns>
      private List<string> GetMergeBlockNames()
      {
         var result = new List<string>();
         foreach (var block in GetMergeBlocks())
         {
            result.Add(block.Name);
         }
         return result;
      }

      /// <summary>
      /// Gathers merge blocks in current document.
      /// </summary>
      /// <returns>Merge blocks in current document.</returns>
      private List<MergeBlock> GetMergeBlocks()
      {
         var result = new List<MergeBlock>();

         // Get a list of all block start and end markers ordered by their start position:
         var blockMarkers = GetBlockMarkersOrdered();

         // Analyze block structure and only gather correctly nested blocks:

         var blocks = new Stack<TXTextControl.DocumentTarget>();

         foreach (var marker in blockMarkers)
         {
            if (marker.TargetName.StartsWith(MergeBlock.BlockStartPrefix, StringComparison.OrdinalIgnoreCase))
            {
               // Start marker:
               blocks.Push(marker);
            }
            else if (marker.TargetName.StartsWith(MergeBlock.BlockEndPrefix, StringComparison.OrdinalIgnoreCase))
            {
               // End marker:

               var strEndMarkerName = marker.TargetName.Substring(MergeBlock.BlockEndPrefix.Length);  // Get block name
               TXTextControl.DocumentTarget startMarker = (blocks.Count > 0) ? blocks.Pop() : null;   // Get currently open start marker
               var strStartMarkerName
                  = (startMarker != null) ? startMarker.TargetName.Substring(MergeBlock.BlockStartPrefix.Length) : string.Empty;

               // Abort if block nesting is invalid
               if (strEndMarkerName.ToLower() != strStartMarkerName.ToLower())
               {
                  if (strStartMarkerName != string.Empty)
                  {
                     MessageBox.Show(
                        "Block nesting invalid somewhere after block start marker “"
                        + MergeBlock.BlockStartPrefix + strStartMarkerName + "”.",
                        ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information
                     );
                  }
                  else
                  {
                     MessageBox.Show(
                        "Invalid block end marker “" + MergeBlock.BlockEndPrefix + strEndMarkerName + "”.",
                        ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information
                     );
                  }
                  break;
               }

               // Create merge block structure and store it
               var blockNew = new MergeBlock { StartMarker = startMarker, EndMarker = marker };
               result.Add(blockNew);
            }
         }

         return result;
      }

      /// <summary>
      /// Returns a list of all block start and end markers ordered by their start position.
      /// </summary>
      /// <returns>Sorted list of block markers.</returns>
      private List<TXTextControl.DocumentTarget> GetBlockMarkersOrdered()
      {
         var targets = new List<TXTextControl.DocumentTarget>();
         foreach (TXTextControl.DocumentTarget tgt in _textControl.DocumentTargets) targets.Add(tgt);

         var blockMarkers = new List<TXTextControl.DocumentTarget>();
         foreach (TXTextControl.DocumentTarget tgt in targets)
         {
            if (tgt.TargetName.StartsWith(MergeBlock.BlockStartPrefix, StringComparison.OrdinalIgnoreCase)
               || tgt.TargetName.StartsWith(MergeBlock.BlockEndPrefix, StringComparison.OrdinalIgnoreCase))
            {
               blockMarkers.Add(tgt);
            }
         }

         blockMarkers.Sort(new DocumentTargetComparer());
         return blockMarkers;
      }

      private void MergePrint()
      {
         // Save current document to memory
         byte[] docData;
         _textControl.Save(out docData, TXTextControl.BinaryStreamType.InternalUnicodeFormat);

         try
         {
            // Merge data
            var mailMerge = new MailMerge() { TextComponent = _textControl };
            mailMerge.LoadTemplateFromMemory(docData, FileFormat.InternalUnicodeFormat);

            // Blocks:
            var mergeBlockTableNames = GetMergeBlockNames();
            if (mergeBlockTableNames.Count > 0)
            {
               mailMerge.MergeBlocks(_dbCon.GetDataSet(mergeBlockTableNames));
            }

            // Normal fields
            var tbl = _dbCon.SelectedTable == null ? null : _dbCon.SelectedTable.DataTable;
            mailMerge.Merge(tbl, true);

            // Show print dialog and print merged document:

            var printDlg = new PrintDialog();
            var printDoc = new PrintDocument();
            printDlg.AllowCurrentPage =
            printDlg.AllowSelection =
            printDlg.AllowSomePages = false;
            printDlg.Document = printDoc;
            if (printDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
               mailMerge.Print(printDlg.Document);
            }
         }
         catch (Exception exc)
         {
            MessageBox.Show(string.Format("Merge failed: {0}", exc.Message, MessageBoxButtons.OK, MessageBoxIcon.Error));
         }
      }

      private void MergePrintAsync()
      {
         ThreadPool.QueueUserWorkItem(MergePrintWaitCallback);
      }

      private void MergePrintWaitCallback(object state)
      {
         // Show merge wait dialog modally in UI thread to block 
         // interaction with main form
         BeginInvoke(new InvokeDelegate(ShowMergeWaitDialog));

         MergePrint();

         // Close wait dialog
         BeginInvoke(new InvokeDelegate(CloseMergeWaitDialog));
      }

      private string _strMergeDocPrefix = string.Empty;
      private string _strMergeDocPath = string.Empty;
      private TXTextControl.StreamType _mergeStreamType = TXTextControl.StreamType.InternalUnicodeFormat;
      private void MergeToMultipleFiles(TXTextControl.StreamType streamType, string strMergeDocPath)
      {
         _strMergeDocPath = strMergeDocPath;

         _mergeStreamType = streamType;
         _strMergeExceptionMessage = string.Empty;
         _nMergedDocs = 0;

         // Generate target document file name prefix from current time
         _strMergeDocPrefix = "MergedDocument_" + string.Format("{0:HHmmss}_", DateTime.Now);

         // Save current document to memory
         byte[] docData;
         _textControl.Save(out docData, TXTextControl.BinaryStreamType.InternalUnicodeFormat);

         try
         {
            // Merge data:

            var mailMerge = new MailMerge { TextComponent = _textControl };
            mailMerge.LoadTemplateFromMemory(docData, FileFormat.InternalUnicodeFormat);

            // Blocks:
            var mergeBlockTableNames = GetMergeBlockNames();
            if (mergeBlockTableNames.Count > 0)
            {
               mailMerge.MergeBlocks(_dbCon.GetDataSet(mergeBlockTableNames));
            }

            // Register event handler which is fired after each merged data row
            mailMerge.DataRowMerged += MailMerge_DataRowMerged;

            // Start merging
            var tbl = _dbCon.SelectedTable == null ? null : _dbCon.SelectedTable.DataTable;
            mailMerge.Merge(tbl, false);

            // Show exception message if merging failed for some reason
            if (_strMergeExceptionMessage.Length > 0)
            {
               MessageBox.Show(
                  string.Format("Merge failed: {0}", _strMergeExceptionMessage), 
                  ProductName, 
                  MessageBoxButtons.OK, 
                  MessageBoxIcon.Error
               );
            }
            else MessageBox.Show(
               string.Format("Merge successful. {0} documents were created.", _nMergedDocs),
               ProductName,
               MessageBoxButtons.OK,
               MessageBoxIcon.Information
            );
         }
         catch (Exception exc)
         {
            MessageBox.Show(string.Format("Merge failed: {0}", exc.Message), ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
         }
      }

      private void MergeToMultipleFilesAsync(TXTextControl.StreamType streamType)
      {
         // Let user select a target directory:
         var fbd = new FolderBrowserDialog();
         if (fbd.ShowDialog() != System.Windows.Forms.DialogResult.OK) return;

         var paramsArray = new object[] { streamType, fbd.SelectedPath };  // Prepare needed parameters
         ThreadPool.QueueUserWorkItem(MergeToMultipleFilesWaitCallback, paramsArray);
      }

      private void MergeToMultipleFilesWaitCallback(object state)
      {
         var paramsArray = state as object[];
         var streamType = (TXTextControl.StreamType) paramsArray[0];
         var selectedPath = paramsArray[1] as string;

         // Show merge wait dialog modally in UI thread to block 
         // interaction with main form
         BeginInvoke(new InvokeDelegate(ShowMergeWaitDialog));

         MergeToMultipleFiles(streamType, selectedPath);

         // Close wait dialog
         BeginInvoke(new InvokeDelegate(CloseMergeWaitDialog));
      }

      private string _strMergeExceptionMessage = string.Empty;
      private int _nMergedDocs;
      void MailMerge_DataRowMerged(object sender, MailMerge.DataRowMergedEventArgs e)
      {
         if (_strMergeExceptionMessage.Length > 0) return;  // Something went wrong

         string strFileName
            = _strMergeDocPath
            + "\\" + _strMergeDocPrefix
            + string.Format("{0:00000}", e.DataRowNumber)
            + "." + StreamTypeToFileExt(_mergeStreamType);

         switch (_mergeStreamType)
         {
            case TXTextControl.StreamType.InternalUnicodeFormat:
               // Just save binary data to disk
               File.WriteAllBytes(strFileName, e.MergedRow);
               break;

            case TXTextControl.StreamType.HTMLFormat:
            case TXTextControl.StreamType.MSWord:
            case TXTextControl.StreamType.PlainText:
            case TXTextControl.StreamType.RichTextFormat:
            case TXTextControl.StreamType.WordprocessingML:
            case TXTextControl.StreamType.AdobePDF:
               // Load document into temporary text control and convert to desired format
               using (TXTextControl.ServerTextControl tc = new TXTextControl.ServerTextControl())
               {
                  tc.Create();
                  tc.Load(e.MergedRow, TXTextControl.BinaryStreamType.InternalUnicodeFormat);

                  try
                  {
                     tc.Save(strFileName, _mergeStreamType, new TXTextControl.SaveSettings());
                     ++_nMergedDocs;
                  }
                  catch (Exception exc) { _strMergeExceptionMessage = exc.Message; }
               }
               break;
         }
      }

      private void MergeSaveSingleDoc(TXTextControl.StreamType streamType, string strFileName)
      {
         // Save current document to memory
         byte[] docData;
         _textControl.Save(out docData, TXTextControl.BinaryStreamType.InternalUnicodeFormat);

         try
         {
            // Merge data
            var mailMerge = new MailMerge { TextComponent = _textControl };
            mailMerge.LoadTemplateFromMemory(docData, FileFormat.InternalUnicodeFormat);

            // Blocks:
            var mergeBlockTableNames = GetMergeBlockNames();
            if (mergeBlockTableNames.Count > 0)
            {
               mailMerge.MergeBlocks(_dbCon.GetDataSet(mergeBlockTableNames));
            }

            // Normal fields:
            var tbl = _dbCon.SelectedTable == null ? null : _dbCon.SelectedTable.DataTable;
            mailMerge.Merge(tbl, true);

            // Save document
            mailMerge.SaveDocument(strFileName, streamType, new TXTextControl.SaveSettings());
         }
         catch (Exception exc)
         {
            MessageBox.Show(string.Format("Merge failed: {0}", exc.Message, MessageBoxButtons.OK, MessageBoxIcon.Error));
         }
      }

      private void MergeSaveSingleDocAsync(TXTextControl.StreamType streamType)
      {
         var sdfd = new SaveDocumentFileDialog { SelectedFileType = streamType };
         if (!sdfd.ShowDialog()) return;

         var paramsArray = new object[] { sdfd.SelectedFileType, sdfd.FileName };
         ThreadPool.QueueUserWorkItem(MergeSavesingleWaitCallback, paramsArray);
      }

      private void MergeSavesingleWaitCallback(object state)
      {
         var paramsArray = state as object[];
         var selectedFileType = (TXTextControl.StreamType) paramsArray[0];
         var fileName = paramsArray[1] as string;

         // Show merge wait dialog modally in UI thread to block 
         // interaction with main form
         BeginInvoke(new InvokeDelegate(ShowMergeWaitDialog));

         MergeSaveSingleDoc(selectedFileType, fileName);

         // Close wait dialog
         BeginInvoke(new InvokeDelegate(CloseMergeWaitDialog));
      }

      private void MergeIntoCurrentDoc()
      {
         // Merge:
         try
         {
            var mailMerge = new MailMerge { TextComponent = _textControl };

            // Blocks:
            var mergeBlockTableNames = GetMergeBlockNames();
            if (mergeBlockTableNames.Count > 0)
            {
               mailMerge.MergeBlocks(_dbCon.GetDataSet(mergeBlockTableNames));
            }

            // Normal fields:
            var tbl = _dbCon.SelectedTable == null ? null : _dbCon.SelectedTable.DataTable;
            mailMerge.Merge(tbl, true);
         }
         catch (Exception exc)
         {
            MessageBox.Show(string.Format("Merge failed: {0}", exc.Message, MessageBoxButtons.OK, MessageBoxIcon.Error));
         }

         _fileHandler.DocumentDirty = true;
      }

      private void MergeIntoCurrentDocAsync()
      {  
         ThreadPool.QueueUserWorkItem(MergeIntoCurrentDocWaitCallback);
      }

      private void MergeIntoCurrentDocWaitCallback(object state)
      {
         // Show merge wait dialog modally in UI thread to block 
         // interaction with main form
         BeginInvoke(new InvokeDelegate(ShowMergeWaitDialog));

         MergeIntoCurrentDoc();

         // Close wait dialog
         BeginInvoke(new InvokeDelegate(CloseMergeWaitDialog));
      }

      /// <summary>
      /// Returns names of all charts in current document.
      /// </summary>
      /// <returns>List of chart names in the current document.</returns>
      private List<string> GetChartNames()
      {
         var result = new List<string>();
         foreach (ChartFrame cf in _textControl.Charts) result.Add(cf.Name);
         return result;
      }

      /// <summary>
      /// Gathers chart frames in current document.
      /// </summary>
      /// <returns>Chart frames in current document</returns>
      private List<ChartFrame> GetCharts()
      {
         var result = new List<ChartFrame>();
         foreach (ChartFrame cf in _textControl.Charts) result.Add(cf);
         return result;
      }

      /// <summary>
      /// Inserts a table row or a paragraph surrounded by a merge block and containing fields named 
      /// after the columns in the table which has the same name as the block.
      /// </summary>
      /// <param name="strName">Block name.</param>
      private void InsertMergeBlockTemplate(string strName)
      {
         var dlg = new InsertMergeBlockDialog(strName, _dbCon, _textControl);
         if (dlg.ShowDialog() != System.Windows.Forms.DialogResult.OK) return;

         switch (dlg.BlockType)
         {
            case InsertMergeBlockDialog.BlockTemplateType.TableRow:
               InsertMergeBlockUsingTable(strName, dlg.SelectedColumnNames);
               break;

            case InsertMergeBlockDialog.BlockTemplateType.PlainParagraph:
               InsertMergeBlockUsingParagraph(strName, dlg.SelectedColumnNames);
               break;
         }
      }

      private void InsertMergeBlockUsingParagraph(string strName, List<string> fieldNames)
      {
         // Add paragraph if not at the beginning of a line
         if (_textControl.InputPosition.Column != 0) _textControl.Selection.Text = Environment.NewLine;

         // Store current position
         int nPos = _textControl.InputPosition.TextPosition;

         // Create one field for each selected column
         for (var i = 0; i < fieldNames.Count; ++i)
         {
            MergeField mfNew = InsertMergeField(fieldNames[i]);
            _textControl.InputPosition
               = new TXTextControl.InputPosition((mfNew.Start + mfNew.Length) - 1, TXTextControl.TextFieldPosition.OutsideTextField);
            if (i == (fieldNames.Count - 1)) break;
            _textControl.Selection.Text = " ";  // Put whitespaces between fields
         }

         // End block paragraph
         _textControl.Selection.Text = Environment.NewLine;

         // Create block end marker
         _textControl.DocumentTargets.Add(new TXTextControl.DocumentTarget(MergeBlock.BlockEndPrefix + strName) { Deleteable = false });

         // Create block start marker
         _textControl.InputPosition = new TXTextControl.InputPosition(nPos, TXTextControl.TextFieldPosition.OutsideTextField);
         _textControl.DocumentTargets.Add(new TXTextControl.DocumentTarget(MergeBlock.BlockStartPrefix + strName) { Deleteable = false });
      }

      private void InsertMergeBlockUsingTable(string strName, List<string> fieldNames)
      {
         int nIDTbl = GenerateUnusedTableID(_textControl);

         // Create table with as many columns as selected field names
         _textControl.Tables.Add(1, fieldNames.Count, nIDTbl);
         TXTextControl.Table tblNew = _textControl.Tables.GetItem(nIDTbl);
         if (tblNew == null) return;

         // Put one field in each table cell
         for (var i = 0; i < tblNew.Columns.Count; ++i)
         {
            tblNew.Cells.GetItem(1, i + 1).Select();
            _textControl.Selection.Length = 0;
            InsertMergeField(fieldNames[i]);
         }

         // Surround table with block:

         // Block start marker
         _textControl.InputPosition = new TXTextControl.InputPosition(tblNew.Cells.GetItem(1, 1).Start - 1, TXTextControl.TextFieldPosition.OutsideTextField);
         _textControl.DocumentTargets.Add(new TXTextControl.DocumentTarget(MergeBlock.BlockStartPrefix + strName) { Deleteable = false });

         // Block end marker
         tblNew.Select();
         _textControl.InputPosition = new TXTextControl.InputPosition(_textControl.Selection.Start + _textControl.Selection.Length);
         _textControl.DocumentTargets.Add(new TXTextControl.DocumentTarget(MergeBlock.BlockEndPrefix + strName) { Deleteable = false });
      }

      private static int GenerateUnusedTableID(TXTextControl.TextControl textControl)
      {
         var rnd = new Random();
         int nResult = GenerateRandomTXTableID();
         var usedTableIDs = new List<int>();

         foreach (TXTextControl.Table tbl in textControl.Tables)
         {
            if (tbl.ID != 0) usedTableIDs.Add(tbl.ID);
         }

         // Find unused random table id
         if (usedTableIDs.Count > 0)
         {
            while (usedTableIDs.Contains(nResult)) nResult = GenerateRandomTXTableID();
         }

         return nResult;
      }

      /// <summary>
      /// Generates a number between 10 and Int16.MaxValue.
      /// </summary>
      /// <returns>A number between 10 and Int16.MaxValue.</returns>
      private static int GenerateRandomTXTableID()
      {
         var rnd = new Random();
         return rnd.Next(10, Int16.MaxValue);
      }

      /// <summary>
      /// Checks if two given text positions can be used to create a merge block.
      /// </summary>
      /// <param name="nStart">Start position.</param>
      /// <param name="nEnd">End position.</param>
      /// <returns>Text positions overlap with an existing merge block.</returns>
      private bool ValidateMergeBlockBoundaries(int nStart, int nEnd)
      {
         var blocks = GetMergeBlocks();
         var block01 = MergeBlock.FindBlockContainingTextPos(blocks, nStart);
         var block02 = MergeBlock.FindBlockContainingTextPos(blocks, nEnd);

         if ((block01 == null) && (block02 == null)) return true;

         // Start and end positions lie inside different 
         // existing merge blocks
         if (block01 != block02) return false;

         // New block is allowed to be inside an existing one
         return true;
      }

      private bool MergePossible()
      {
         if (_dbCon == null)
         {
            MessageBox.Show(MsgNoDataSource, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
            return false;
         }
         if (_dbCon.SelectedTable == null)
         {
            MessageBox.Show(MsgNoTableSelected, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
            return false;
         }
         if (!DocumentContainsFields())
         {
            MessageBox.Show(MsgNoFields, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
            return false;
         }

         return true;
      }

      #region "  Helpers  "

      private void ShowMergeWaitDialog()
      {
         _dlgMergeWait.ShowDialog();
      }

      private void CloseMergeWaitDialog()
      {
         _dlgMergeWait.Close();
      }

      #endregion // Helpers
   }
}
