﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				MergeBlock.cs
** description:	This file contains a class representing a merge block.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System.Collections.Generic;
using System.Collections;


namespace TX_Text_Control_Words
{
   /// <summary>
   /// Holds two document targets which surround a merge block.
   /// </summary>
   public class MergeBlock
   {
      public TXTextControl.DocumentTarget StartMarker;
      public TXTextControl.DocumentTarget EndMarker;

      public const string BlockStartPrefix = "BlockStart_";
      public const string BlockEndPrefix = "BlockEnd_";

      /// <summary>
      /// Gets the block name by removing the block name prefix from 
      /// the start marker's name.
      /// </summary>
      public string Name
      {
         get
         {
            if (StartMarker == null) return string.Empty;
            return StartMarker.TargetName.Substring(BlockStartPrefix.Length);
         }
      }

      /// <summary>
      /// Returns a merge block containing a given text position in a list of merge blocks. Returns the
      /// innermost nested block in case the text position lies inside nested blocks.
      /// </summary>
      /// <param name="blocks">List of merge blocks.</param>
      /// <param name="nTextPos">Text position</param>
      /// <returns>Block containing the text position or null.</returns>
      public static MergeBlock FindBlockContainingTextPos(List<MergeBlock> blocks, int nTextPos)
      {
         // Get all merge blocks containing the given text position
         var candidates = blocks.FindAll(block => block.Contains(nTextPos));

         if (candidates.Count == 0) return null;
         if (candidates.Count == 1) return candidates[0];

         // Sort blocks according to their length
         candidates.Sort(MergeBlock.SortLength);

         // Return the shortest, i. e. the innermost nested block which contains the text position
         return candidates[0];
      }

      /// <summary>
      /// Returns IComparer comparing block lengths.
      /// </summary>
      /// <returns></returns>
      public static IComparer<MergeBlock> SortLength
      {
         get { return (IComparer<MergeBlock>) new MergeBlockComparer(); }
      }

      /// <summary>
      /// Checks if a text position lies between the block's start and end markers.
      /// </summary>
      /// <param name="nTextPos">The text position.</param>
      /// <returns>The text position is inside the block.</returns>
      private bool Contains(int nTextPos)
      {
         if ((StartMarker == null) || (EndMarker == null)) return false;
         return ((nTextPos >= StartMarker.Start) && (nTextPos <= EndMarker.Start));
      }

      /// <summary>
      /// Gets the block's length in characters.
      /// </summary>
      public int Length
      {
         get
         {
            if ((StartMarker == null) || (EndMarker == null)) return 0;
            return EndMarker.Start - StartMarker.Start;
         }
      }
   } // class MergeBlock
}
