﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				TabGrp_ObjectTools.cs
** description:	This file contains event handlers, fields and helper methods concerning the “Object Tools” 
**                contextual ribbon tab group.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System.Windows.Forms;
using RibbonLib.Controls;
using RibbonLib.Controls.Events;
using RibbonLib.Interop;
using System.Diagnostics;
using System.Reflection;
using System;


namespace TX_Text_Control_Words
{
   public partial class FormMain : Form
   {
      #region "  Private Fields  "

      private RibbonButton _btnObjectProperties;
      private RibbonButton _btnObjArrangeSendToBack;
      private RibbonButton _btnObjArrangeSendBackward;
      private RibbonButton _btnObjArrangeBringForward;
      private RibbonButton _btnObjArrangeBringToFront;
      private RibbonButton _btnObjArrangeBehindText;
      private RibbonButton _btnObjArrangeInFrontOfText;
      private RibbonButton _btnObjArrangeAlignLeft;
      private RibbonButton _btnObjArrangeAlignCenter;
      private RibbonButton _btnObjArrangeAlignRight;
      private RibbonDropDownButton _drpDnObjArrangeTextWrap;
      private RibbonToggleButton _tglBtnObjArrangeWrapInLine;
      private RibbonToggleButton _tglBtnObjArrangeWrapTopBot;
      private RibbonToggleButton _tglBtnObjArrangeWrapSquare;
      private RibbonToggleButton _tglBtnObjArrangeWrapBehind;
      private RibbonToggleButton _tglBtnObjArrangeWrapInFront;
      private RibbonButton _btnObjectName;
      private RibbonButton _btnObjectID;

      #endregion // Private fields

      #region "  Enumerations  "

      enum RibbonMarkupCommand_TabGrpObjectTools
      {
         cmdTabGrpObjectTools = 9000,
         cmdTabObjTlsFormat = 9001,
         cmdGrpObjArrange = 9002,
         cmdBtnObjArrangeSendToBack = 9003,
         cmdBtnObjArrangeSendBackward = 9004,
         cmdBtnObjArrangeBringForward = 9005,
         cmdBtnObjArrangeBringToFront = 9006,
         cmdBtnObjArrangeBehindText = 9007,
         cmdBtnObjArrangeInFrontOfText = 9008,
         cmdDrpDnObjArrangeAlign = 9009,
         cmdBtnObjArrangeAlignLeft = 9010,
         cmdBtnObjArrangeAlignCenter = 9011,
         cmdBtnObjArrangeAlignRight = 9012,
         cmdDrpDnObjArrangeTextWrap = 9013,
         cmdTglBtnObjArrangeWrapInLine = 9014,
         cmdTglBtnObjArrangeWrapTopBot = 9015,
         cmdTglBtnObjArrangeWrapSquare = 9016,
         cmdTglBtnObjArrangeWrapBehind = 9017,
         cmdTglBtnObjArrangeWrapInFront = 9018,
         cmdGrpObjFormat = 9019,
         cmdBtnObjectProperties = 9020,
         cmdBtnObjectName = 9021,
         cmdBtnObjectID = 9022,
      } // enum RibbonMarkupCommand_TabGrpObjectTools

      #endregion // Enumerations

      #region "  Ribbon event handlers  "

      void BtnObjectProperties_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         // Clicking a button in the Ribbon deselects the text control
         // so the currently selected object can be null at this point
         var obj = _objSel ?? _objSelPrev;
         if (obj == null) return;

         try
         {
            if (obj is TXTextControl.Image)
            {
               if (_textControl.ImageAttributesDialog() == DialogResult.OK)
               {
                  _fileHandler.DocumentDirty = true;
               }
               return;
            }

            if (obj is TXTextControl.TextFrame)
            {
               if (_textControl.TextFrameAttributesDialog() == DialogResult.OK)
               {
                  _fileHandler.DocumentDirty = true;
               }
               return;
            }

            if (obj is TXTextControl.DataVisualization.ChartFrame)
            {
               if (_textControl.ChartLayoutDialog() == DialogResult.OK)
               {
                  _fileHandler.DocumentDirty = true;
               }
               return;
            }

            if (obj is TXTextControl.DataVisualization.BarcodeFrame)
            {
               if (_textControl.BarcodeLayoutDialog() == DialogResult.OK)
               {
                  _fileHandler.DocumentDirty = true;
               }
               return;
            }
         }
         catch { }
      }

      void SetSelectedObjHorAlign(TXTextControl.HorizontalAlignment align)
      {
         // Clicking a button in the Ribbon deselects the text control
         // so the currently selected object can be null at this point
         var obj = _objSel ?? _objSelPrev;
         if (obj == null) return;

         obj.Alignment = align;
      }

      void BtnObjArrangeAlignRight_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         SetSelectedObjHorAlign(TXTextControl.HorizontalAlignment.Right);
      }

      void BtnObjArrangeAlignCenter_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         SetSelectedObjHorAlign(TXTextControl.HorizontalAlignment.Center);
      }

      void BtnObjArrangeAlignLeft_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         SetSelectedObjHorAlign(TXTextControl.HorizontalAlignment.Left);
      }

      void BtnObjArrangeInFrontOfText_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         UntoggleTextWrapTglBtns();
         SetObjectInsertionMode(_objSel, TXTextControl.FrameInsertionMode.AboveTheText);
      }

      void BtnObjArrangeBehindText_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         UntoggleTextWrapTglBtns();
         SetObjectInsertionMode(_objSel, TXTextControl.FrameInsertionMode.BelowTheText);
      }

      private void SetSelectedObjZOrder(TXTextControl.ZOrder zOrder)
      {
         var obj = _objSel ?? _objSelPrev;
         if (obj == null) return;

         obj.ChangeZOrder(zOrder);
      }

      void BtnObjArrangeBringToFront_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         SetSelectedObjZOrder(TXTextControl.ZOrder.Top);
      }

      void BtnObjArrangeBringForward_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         SetSelectedObjZOrder(TXTextControl.ZOrder.Up);
      }

      void BtnObjArrangeSendBackward_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         SetSelectedObjZOrder(TXTextControl.ZOrder.Down);
      }

      void BtnObjArrangeSendToBack_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         SetSelectedObjZOrder(TXTextControl.ZOrder.Bottom);
      }

      void TglBtnObjArrangeWrapInFront_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         // Clicking a button in the Ribbon deselects the text control
         // so the currently selected object can be null at this point
         var obj = _objSel ?? _objSelPrev;
         if (obj == null) return;

         UntoggleTextWrapTglBtns();
         _tglBtnObjArrangeWrapInFront.BooleanValue = true;
         SetObjectInsertionMode(obj, TXTextControl.FrameInsertionMode.AboveTheText);
      }

      void TglBtnObjArrangeWrapBehind_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         // Clicking a button in the Ribbon deselects the text control
         // so the currently selected object can be null at this point
         var obj = _objSel ?? _objSelPrev;
         if (obj == null) return;

         UntoggleTextWrapTglBtns();
         _tglBtnObjArrangeWrapBehind.BooleanValue = true;
         SetObjectInsertionMode(obj, TXTextControl.FrameInsertionMode.BelowTheText);
      }

      void TglBtnObjArrangeWrapSquare_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         // Clicking a button in the Ribbon deselects the text control
         // so the currently selected object can be null at this point
         var obj = _objSel ?? _objSelPrev;
         if (obj == null) return;

         UntoggleTextWrapTglBtns();
         _tglBtnObjArrangeWrapSquare.BooleanValue = true;
         SetObjectInsertionMode(obj, TXTextControl.FrameInsertionMode.DisplaceText);
      }

      void TglBtnObjArrangeWrapTopBot_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         // Clicking a button in the Ribbon deselects the text control
         // so the currently selected object can be null at this point
         var obj = _objSel ?? _objSelPrev;
         if (obj == null) return;

         UntoggleTextWrapTglBtns();
         _tglBtnObjArrangeWrapTopBot.BooleanValue = true;
         SetObjectInsertionMode(obj, TXTextControl.FrameInsertionMode.DisplaceCompleteLines);
      }

      void TglBtnObjArrangeWrapInLine_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         // Clicking a button in the Ribbon deselects the text control
         // so the currently selected object can be null at this point
         var obj = _objSel ?? _objSelPrev;
         if (obj == null) return;

         UntoggleTextWrapTglBtns();
         _tglBtnObjArrangeWrapInLine.BooleanValue = true;
         SetObjectInsertionMode(obj, TXTextControl.FrameInsertionMode.AsCharacter);
      }

      void BtnObjectID_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         // Clicking a button in the Ribbon deselects the text control
         // so the currently selected object can be null at this point
         var obj = _objSel ?? _objSelPrev;

         SetObjectID(obj);
      }

      void BtnObjectName_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         // Clicking a button in the Ribbon deselects the text control
         // so the currently selected object can be null at this point
         var obj = _objSel ?? _objSelPrev;

         SetObjectName(obj);
      }

      #endregion // Ribbon event handlers

      #region "  Helpers  "

      private void CreateRibbonItems_TabGrpObjectTools()
      {
         _btnObjectProperties = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpObjectTools.cmdBtnObjectProperties);
         _btnObjArrangeSendToBack = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpObjectTools.cmdBtnObjArrangeSendToBack);
         _btnObjArrangeSendBackward = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpObjectTools.cmdBtnObjArrangeSendBackward);
         _btnObjArrangeBringForward = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpObjectTools.cmdBtnObjArrangeBringForward);
         _btnObjArrangeBringToFront = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpObjectTools.cmdBtnObjArrangeBringToFront);
         _btnObjArrangeBehindText = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpObjectTools.cmdBtnObjArrangeBehindText);
         _btnObjArrangeInFrontOfText = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpObjectTools.cmdBtnObjArrangeInFrontOfText);
         _btnObjArrangeAlignLeft = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpObjectTools.cmdBtnObjArrangeAlignLeft);
         _btnObjArrangeAlignCenter = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpObjectTools.cmdBtnObjArrangeAlignCenter);
         _btnObjArrangeAlignRight = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpObjectTools.cmdBtnObjArrangeAlignRight);
         _drpDnObjArrangeTextWrap = new RibbonDropDownButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpObjectTools.cmdDrpDnObjArrangeTextWrap);
         _tglBtnObjArrangeWrapInLine = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpObjectTools.cmdTglBtnObjArrangeWrapInLine);
         _tglBtnObjArrangeWrapTopBot = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpObjectTools.cmdTglBtnObjArrangeWrapTopBot);
         _tglBtnObjArrangeWrapSquare = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpObjectTools.cmdTglBtnObjArrangeWrapSquare);
         _tglBtnObjArrangeWrapBehind = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpObjectTools.cmdTglBtnObjArrangeWrapBehind);
         _tglBtnObjArrangeWrapInFront = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabGrpObjectTools.cmdTglBtnObjArrangeWrapInFront);
         _btnObjectName = new RibbonButton(_ribbonMain, (uint) RibbonMarkupCommand_TabGrpObjectTools.cmdBtnObjectName);
         _btnObjectID = new RibbonButton(_ribbonMain, (uint) RibbonMarkupCommand_TabGrpObjectTools.cmdBtnObjectID);
      }

      private void AttachRibbonEvents_TabGrpObjectTools()
      {
         _btnObjectProperties.ExecuteEvent += BtnObjectProperties_ExecuteEvent;
         _btnObjArrangeSendToBack.ExecuteEvent += BtnObjArrangeSendToBack_ExecuteEvent;
         _btnObjArrangeSendBackward.ExecuteEvent += BtnObjArrangeSendBackward_ExecuteEvent;
         _btnObjArrangeBringForward.ExecuteEvent += BtnObjArrangeBringForward_ExecuteEvent;
         _btnObjArrangeBringToFront.ExecuteEvent += BtnObjArrangeBringToFront_ExecuteEvent;
         _btnObjArrangeBehindText.ExecuteEvent += BtnObjArrangeBehindText_ExecuteEvent;
         _btnObjArrangeInFrontOfText.ExecuteEvent += BtnObjArrangeInFrontOfText_ExecuteEvent;
         _btnObjArrangeAlignLeft.ExecuteEvent += BtnObjArrangeAlignLeft_ExecuteEvent;
         _btnObjArrangeAlignCenter.ExecuteEvent += BtnObjArrangeAlignCenter_ExecuteEvent;
         _btnObjArrangeAlignRight.ExecuteEvent += BtnObjArrangeAlignRight_ExecuteEvent;
         _tglBtnObjArrangeWrapInLine.ExecuteEvent += TglBtnObjArrangeWrapInLine_ExecuteEvent;
         _tglBtnObjArrangeWrapTopBot.ExecuteEvent += TglBtnObjArrangeWrapTopBot_ExecuteEvent;
         _tglBtnObjArrangeWrapSquare.ExecuteEvent += TglBtnObjArrangeWrapSquare_ExecuteEvent;
         _tglBtnObjArrangeWrapBehind.ExecuteEvent += TglBtnObjArrangeWrapBehind_ExecuteEvent;
         _tglBtnObjArrangeWrapInFront.ExecuteEvent += TglBtnObjArrangeWrapInFront_ExecuteEvent;
         _btnObjectName.ExecuteEvent += BtnObjectName_ExecuteEvent;
         _btnObjectID.ExecuteEvent += BtnObjectID_ExecuteEvent;
      }

      /// <summary>
      /// Checks if an object (i. e. a text frame, a chart or an image) which is not inserted
      /// as a character is selected.
      /// </summary>
      /// <returns>An object which is not inserted as a character is selected.</returns>
      private bool IsAnchoredOrFixedObjectSelected()
      {
         if (_objSel == null) return false;

         uint uInsMode = (uint)(_objSel.InsertionMode & ~(TXTextControl.FrameInsertionMode.FixedOnPage | TXTextControl.FrameInsertionMode.MoveWithText));
         return (uInsMode != (uint)TXTextControl.FrameInsertionMode.AsCharacter);
      }

      private void UpdateContextualObjectTabGroup()
      {
         _tabGroupObjectTools.ContextAvailable
            = IsAnchoredOrFixedObjectSelected() ? ContextAvailability.Available : ContextAvailability.NotAvailable;

         UpdateTextWrapToggleBtns();
      }

      private void GetSelectedObject()
      {
         _objSelPrev = _objSel;
         _objSel = null;

         try
         {
            _objSel = _textControl.Frames.GetItem();
         }
         catch { }

      }

      private TXTextControl.FrameInsertionMode GetSelectedObjInsMode()
      {
         if (_objSel == null) return TXTextControl.FrameInsertionMode.AsCharacter;
         return _objSel.InsertionMode & ~(TXTextControl.FrameInsertionMode.MoveWithText | TXTextControl.FrameInsertionMode.FixedOnPage);
      }

      private void UpdateTextWrapToggleBtns()
      {
         UntoggleTextWrapTglBtns();

         switch (GetSelectedObjInsMode())
         {
            case TXTextControl.FrameInsertionMode.AsCharacter:
               _tglBtnObjArrangeWrapInLine.BooleanValue = true;
               break;

            case TXTextControl.FrameInsertionMode.AboveTheText:
               _tglBtnObjArrangeWrapInFront.BooleanValue = true;
               break;

            case TXTextControl.FrameInsertionMode.DisplaceCompleteLines:
               _tglBtnObjArrangeWrapTopBot.BooleanValue = true;
               break;

            case TXTextControl.FrameInsertionMode.DisplaceText:
               _tglBtnObjArrangeWrapSquare.BooleanValue = true;
               break;

            case TXTextControl.FrameInsertionMode.BelowTheText:
               _tglBtnObjArrangeWrapBehind.BooleanValue = true;
               break;
         }
      }

      void SetSelectedObjectInsertionMode(object mnuItem, TXTextControl.FrameInsertionMode insMode)
      {
         ToolStripMenuItem mi = mnuItem as ToolStripMenuItem;
         if ((mi == null) || mi.Checked) return;

         // Set new insertion mode
         SetObjectInsertionMode(_objSel, insMode);
         mi.Checked = true;
      }

      void SetObjectInsertionMode(TXTextControl.FrameBase obj, TXTextControl.FrameInsertionMode insMode)
      {
         if (obj == null) return;

         // If new insertion mode is "as character" discard previous insertion mode flags
         if (insMode == TXTextControl.FrameInsertionMode.AsCharacter)
         {
            obj.InsertionMode = insMode;

            // Activate or deactivate contextual tab
            UpdateContextualObjectTabGroup();
            return;
         }

         // Get current insertion mode flags
         TXTextControl.FrameInsertionMode insModeFlags
            = (obj.InsertionMode & (TXTextControl.FrameInsertionMode.FixedOnPage | TXTextControl.FrameInsertionMode.MoveWithText));

         // If currently no insertion mode flags are set, set to "move with text"
         if (insModeFlags == (TXTextControl.FrameInsertionMode)0)
         {
            insModeFlags = TXTextControl.FrameInsertionMode.MoveWithText;
         }

         // Combine new insertion mode with current insertion mode flags
         obj.InsertionMode = insModeFlags | insMode;

         // Activate or deactivate contextual tab
         UpdateContextualObjectTabGroup();
      }

      private void UntoggleTextWrapTglBtns()
      {
         _tglBtnObjArrangeWrapBehind.BooleanValue =
         _tglBtnObjArrangeWrapInFront.BooleanValue =
         _tglBtnObjArrangeWrapInLine.BooleanValue =
         _tglBtnObjArrangeWrapSquare.BooleanValue =
         _tglBtnObjArrangeWrapTopBot.BooleanValue = false;
      }

      private void SetObjectID(TXTextControl.FrameBase obj)
      {
         if (obj == null) return;

         // Get current ID
         string strID = "";
         if (obj.ID > 0) strID = obj.ID.ToString();

         // Show input box
         if (FrmInputBox.ShowInputBox("Object ID", ref strID))
         {
            int nID;
            if (int.TryParse(strID, out nID))
            {
               try
               {
                  // Set new ID
                  obj.ID = nID;
               }
               catch (Exception exc)
               {
                  MessageBox.Show(exc.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
               }
            }
            else
            {
               MessageBox.Show("Please enter a valid integer.", ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
         }
      }

      private void SetObjectName(TXTextControl.FrameBase obj)
      {
         if (obj == null) return;

         string strName = obj.Name;
         if (FrmInputBox.ShowInputBox("Object Name", ref strName))
         {
            obj.Name = strName;
         }
      }
      
      #endregion // Helpers
   }
}
