﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				Tab_Home.cs
** description:	This file contains event handlers, fields and helper methods concerning the “Home” 
**                ribbon tab.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using RibbonLib;
using RibbonLib.Controls;
using RibbonLib.Controls.Events;
using RibbonLib.Interop;
using System.Diagnostics;


namespace TX_Text_Control_Words
{
   public partial class FormMain : Form
   {
      #region "  Nested classes  "

      internal struct FontControlState
      {
         [Flags]
         public enum Difference : ushort
         {
            None,

            Family = 0x0001,
            Size = 0x0002,
            Bold = 0x0004,
            Italic = 0x0008,
            Underline = 0x0010,
            Strikethrough = 0x0020,
            VerticalPositioning = 0x0040,
            BackgroundColor = 0x0080,
            ForegroundColor = 0x0100,
         }

         public string Family;
         public decimal Size;
         public FontProperties Bold;
         public FontProperties Italic;
         public FontUnderline Underline;
         public FontProperties Strikethrough;
         public FontVerticalPosition VerticalPositioning;
         public Color BackgroundColor;
         public Color ForegroundColor;

         /// <summary>
         /// Copies font properties from a ribbon font control object.
         /// </summary>
         /// <param name="fntCntrl">Ribbon font control.</param>
         public void CopyValues(RibbonFontControl fntCntrl)
         {
            Family = fntCntrl.Family;
            Size = fntCntrl.Size;
            Bold = fntCntrl.Bold;
            Italic = fntCntrl.Italic;
            Underline = fntCntrl.Underline;
            Strikethrough = fntCntrl.Strikethrough;
            VerticalPositioning = fntCntrl.VerticalPositioning;
            BackgroundColor = fntCntrl.BackgroundColor;
            ForegroundColor = fntCntrl.ForegroundColor;
         }

         /// <summary>
         /// Compares a ribbon font control object to stored font properties.
         /// </summary>
         /// <param name="fntCntrl">Ribbon font control.</param>
         /// <returns>Difference.</returns>
         public Difference CompareTo(RibbonFontControl fntCntrl)
         {
            Difference result = Difference.None;

            if (fntCntrl.Family != Family) result |= Difference.Family;
            if (fntCntrl.Size != Size) result |= Difference.Size;
            if (fntCntrl.Bold != Bold) result |= Difference.Bold;
            if (fntCntrl.Italic != Italic) result |= Difference.Italic;
            if (fntCntrl.Underline != Underline) result |= Difference.Underline;
            if (fntCntrl.Strikethrough != Strikethrough) result |= Difference.Strikethrough;
            if (fntCntrl.VerticalPositioning != VerticalPositioning) result |= Difference.VerticalPositioning;
            if (fntCntrl.BackgroundColor.ToArgb() != BackgroundColor.ToArgb()) result |= Difference.BackgroundColor;
            if (fntCntrl.ForegroundColor.ToArgb() != ForegroundColor.ToArgb()) result |= Difference.ForegroundColor;

            return result;
         }
      } // struct FontControlState

      #endregion

      #region "  Private fields  "

      private FontControlState _fntCntrlState;

      private RibbonButton _btnPaste;
      private RibbonButton _btnCut;
      private RibbonButton _btnCopy;
      private RibbonButton _btnPasteSpecial;
      private RibbonGroup _grpClipboard;
      private RibbonTab _tabHome;
      private RibbonButton _btnFmtBullets;
      private RibbonButton _btnFmtNumbering;
      private RibbonButton _btnFmtStructured;
      private RibbonButton _btnIndIncr;
      private RibbonButton _btnIndDecr;
      private RibbonButton _btnTabPos;
      private RibbonSplitButtonGallery _spltBtnGlryStyles;
      private RibbonButton _btnManageStyles;
      private RibbonSplitButtonGallery _spltBtnGlryUL;
      private RibbonSplitButtonGallery _spltBtnGlryOL;
      private RibbonSplitButtonGallery _spltBtnGlryMultilvl;
      private RibbonToggleButton _tglBtnCntrlChar;
      private RibbonToggleButton[] _tglBtnsHorAlign;     // See also enum HorAlignButton
      private RibbonCheckBox[] _chkBoxesLineSpace;       // See also enum LineSpaceButton
      private RibbonToggleButton[] _tglBtnsBorder;       // See also enum BorderButton
      private RibbonButton _btnParFormat;
      private RibbonFontControl _fntCntrl;
      private RibbonButton _btnFind;
      private RibbonButton _btnReplace;
      private RibbonButton _btnGoTo;
      private bool _bInFontCnrlPreviewEvent;

      #endregion // Private fields

      #region "  Enumerations  "

      public enum RibbonMarkupCommand_TabHome : uint
      {
         cmdTabHome = 1001,
         cmdGrpClipboard = 1002,
         cmdSpltBtnPaste = 1055,
         cmdBtnPaste = 1003,
         cmdBtnCut = 1004,
         cmdBtnCopy = 1005,
         cmdBtnPasteSpecial = 1006,
         cmdGrpFont = 1007,
         cmdGrpParagraph = 1008,
         cmdBtnLstUnordered = 1009,
         cmdBtnLstOrdered = 1010,
         cmdBtnLstMultiLvl = 1011,
         cmdTglBtnULBullet = 1012,
         cmdTglBtnULDash = 1013,
         cmdTglBtnULAsterisk = 1014,
         cmdTglBtnULDiamond = 1015,
         cmdTglBtnULArrow = 1016,
         cmdBtnFmtBullets = 1017,
         cmdBtnFmtNumbering = 1018,
         cmdBtnFmtMultiLvl = 1019,
         cmdBtnIndDecr = 1020,
         cmdBtnIndIncr = 1021,
         cmdBtnTabPos = 1022,
         cmdTglBtnCntrlChar = 1023,
         cmdSpltBtnGlryStyles = 1024,
         cmdGrpStyles = 1025,
         cmdBtnManageStyles = 1026,
         cmdGrpEditing = 1027,
         cmdBtnFind = 1028,
         cmdBtnReplace = 1029,
         cmdBtnGoTo = 1030,
         cmdTglBtnHorAlignLeft = 1031,
         cmdTglBtnHorAlignCenter = 1032,
         cmdTglBtnHorAlignRight = 1033,
         cmdTglBtnHorAlignJust = 1034,
         cmdDrpDnBtnLnSpace = 1035,
         cmdChkBoxLnSpace_1_00 = 1036,
         cmdChkBoxLnSpace_1_15 = 1037,
         cmdChkBoxLnSpace_1_50 = 1038,
         cmdChkBoxLnSpace_2_00 = 1039,
         cmdChkBoxLnSpace_2_50 = 1040,
         cmdChkBoxLnSpace_3_00 = 1041,
         cmdSpltBtnBorders = 1056,
         cmdTglBtnBorderLeft = 1043,
         cmdTglBtnBorderTop = 1044,
         cmdTglBtnBorderRight = 1045,
         cmdTglBtnBorderBottom = 1046,
         cmdTglBtnBorderOutside = 1047,
         cmdTglBtnBorderAll = 1048,
         cmdTglBtnBorderInsideHor = 1049,
         cmdTglBtnBorderInsideVert = 1050,
         cmdBtnParFormat = 1051,
         cmdFntCntrol = 1052,
      }

      /// <summary>
      /// Positions of unordered list gallery buttons in same order as 
      /// images in _imgLstGalleryUL.
      /// </summary>
      public enum UnorderedListGalleryButton : uint
      {
         Bullet,
         Dash,
         Asterisk,
         Diamond,
         Arrow,
      }

      /// <summary>
      /// Positions of numbered list gallery buttons in same order as 
      /// images in _imgLstGalleryOL.
      /// </summary>
      public enum OrderedListGalleryButton : uint
      {
         _123,
         ABC,
         abc,
         I_II_III,
         i_ii_iii,
      }

      /// <summary>
      /// Command IDs for structured list gallery buttons in same order as 
      /// images in _imgLstGalleryMultiLvl.
      /// </summary>
      public enum MultiLvlListGalleryButton : uint
      {
         _123,
         ABC,
         abc,
         I_II_III,
         i_ii_iii,
      }

      private enum HorAlignButton
      {
         Left,
         Center,
         Right,
         Justify,
      }

      private enum LineSpaceChkBox
      {
         _1_00,
         _1_15,
         _1_50,
         _2_00,
         _2_50,
         _3_00,
      }

      private enum BorderButton
      {
         Left,
         Top,
         Right,
         Bottom,
         Outside,
         All,
         InsideHor,
         InsideVert,
      }

      #endregion // Enumerations

      #region "  Text Control event handlers  "

      void InputFormat_TopFrameLineChanged(object sender, EventArgs e)
      {
         RibbonToggleButton btn = _tglBtnsBorder[(int)BorderButton.Top];
         btn.BooleanValue = _textControl.InputFormat.TopFrameLine;
      }

      void InputFormat_StructuredListFormatChanged(object sender, EventArgs e)
      {
         RibbonSplitButtonGallery glry = _spltBtnGlryMultilvl;
         var fmt = _textControl.InputFormat.StructuredListFormat;
         bool isToggled = (fmt != TXTextControl.NumberFormat.None);

         SelectNumberedListGalleryItem(glry, isToggled, fmt);
      }

      void InputFormat_StructuredListChanged(object sender, EventArgs e)
      {
         _spltBtnGlryMultilvl.BooleanValue = _textControl.InputFormat.StructuredList ?? false;
      }

      private static void SelectNumberedListGalleryItem(RibbonSplitButtonGallery glry, bool isToggled, TXTextControl.NumberFormat fmt)
      {
         glry.BooleanValue = isToggled;

         switch (fmt)
         {
            case TXTextControl.NumberFormat.ArabicNumbers:
               glry.SelectedItem = (uint)MultiLvlListGalleryButton._123;
               break;

            case TXTextControl.NumberFormat.CapitalLetters:
               glry.SelectedItem = (uint)MultiLvlListGalleryButton.ABC;
               break;

            case TXTextControl.NumberFormat.Letters:
               glry.SelectedItem = (uint)MultiLvlListGalleryButton.abc;
               break;

            case TXTextControl.NumberFormat.None:
               glry.SelectedItem = uint.MaxValue;
               break;

            case TXTextControl.NumberFormat.RomanNumbers:
               glry.SelectedItem = (uint)MultiLvlListGalleryButton.I_II_III;
               break;

            case TXTextControl.NumberFormat.SmallRomanNumbers:
               glry.SelectedItem = (uint)MultiLvlListGalleryButton.i_ii_iii;
               break;
         }
      }

      void InputFormat_RightFrameLineChanged(object sender, EventArgs e)
      {
         var btn = _tglBtnsBorder[(int)BorderButton.Right];
         btn.BooleanValue = _textControl.InputFormat.RightFrameLine;
      }

      void InputFormat_RightAlignedChanged(object sender, EventArgs e)
      {
         var btn = _tglBtnsHorAlign[(int)HorAlignButton.Right];

         if (_textControl.InputFormat.RightAligned == null) btn.BooleanValue = false;
         else btn.BooleanValue = (_textControl.InputFormat.RightAligned == true);
      }

      void InputFormat_NumberedListFormatChanged(object sender, EventArgs e)
      {
         var glry = _spltBtnGlryOL;
         var fmt = _textControl.InputFormat.NumberedListFormat;
         bool isToggled = (fmt != TXTextControl.NumberFormat.None);

         SelectNumberedListGalleryItem(glry, isToggled, fmt);
      }

      void InputFormat_NumberedListChanged(object sender, EventArgs e)
      {
         _spltBtnGlryOL.BooleanValue = _textControl.InputFormat.NumberedList ?? false;
      }

      void InputFormat_LineSpacingChanged(object sender, EventArgs e)
      {
         RibbonCheckBox chk = null;

         switch (_textControl.InputFormat.LineSpacing)
         {
            case 100:
               chk = _chkBoxesLineSpace[(int)LineSpaceChkBox._1_00];
               break;

            case 115:
               chk = _chkBoxesLineSpace[(int)LineSpaceChkBox._1_15];
               break;

            case 150:
               chk = _chkBoxesLineSpace[(int)LineSpaceChkBox._1_50];
               break;

            case 200:
               chk = _chkBoxesLineSpace[(int)LineSpaceChkBox._2_00];
               break;

            case 250:
               chk = _chkBoxesLineSpace[(int)LineSpaceChkBox._2_50];
               break;

            case 300:
               chk = _chkBoxesLineSpace[(int)LineSpaceChkBox._3_00];
               break;
         }

         UntoggleCheckboxes(_chkBoxesLineSpace, chk);
         if (chk != null) chk.BooleanValue = true;
      }

      void InputFormat_LeftFrameLineChanged(object sender, EventArgs e)
      {
         var btn = _tglBtnsBorder[(int)BorderButton.Left];
         btn.BooleanValue = _textControl.InputFormat.LeftFrameLine;
      }

      void InputFormat_LeftAlignedChanged(object sender, EventArgs e)
      {
         var btn = _tglBtnsHorAlign[(int)HorAlignButton.Left];

         if (_textControl.InputFormat.LeftAligned == null) btn.BooleanValue = false;
         else btn.BooleanValue = (_textControl.InputFormat.LeftAligned == true);
      }

      void InputFormat_JustifiedChanged(object sender, EventArgs e)
      {
         var btn = _tglBtnsHorAlign[(int)HorAlignButton.Justify];

         if (_textControl.InputFormat.Justified == null) btn.BooleanValue = false;
         else btn.BooleanValue = (_textControl.InputFormat.Justified == true);
      }

      void InputFormat_InnerVerticalFrameLinesChanged(object sender, EventArgs e)
      {
         var btn = _tglBtnsBorder[(int)BorderButton.InsideVert];
         btn.BooleanValue = _textControl.InputFormat.InnerVerticalFrameLines;
      }

      void InputFormat_InnerHorizontalFrameLinesChanged(object sender, EventArgs e)
      {
         var btn = _tglBtnsBorder[(int)BorderButton.InsideHor];
         btn.BooleanValue = _textControl.InputFormat.InnerHorizontalFrameLines;
      }

      void InputFormat_CenteredChanged(object sender, EventArgs e)
      {
         var btn = _tglBtnsHorAlign[(int)HorAlignButton.Center];

         if (_textControl.InputFormat.Centered == null) btn.BooleanValue = false;            
         else btn.BooleanValue = (_textControl.InputFormat.Centered == true);
      }

      void InputFormat_BulletedListChanged(object sender, EventArgs e)
      {
         _spltBtnGlryUL.BooleanValue = _textControl.InputFormat.BulletedList ?? false;
      }

      void InputFormat_BulletCharacterChanged(object sender, EventArgs e)
      {
         switch (_textControl.InputFormat.BulletCharacter)
         {
            case (char)0xB7:
               _spltBtnGlryUL.SelectedItem = (uint)UnorderedListGalleryButton.Bullet;
               break;

            case (char)0x2D:
               _spltBtnGlryUL.SelectedItem = (uint)UnorderedListGalleryButton.Dash;
               break;

            case (char)0x2A:
               _spltBtnGlryUL.SelectedItem = (uint)UnorderedListGalleryButton.Asterisk;
               break;

            case (char)0xA8:
               _spltBtnGlryUL.SelectedItem = (uint)UnorderedListGalleryButton.Diamond;
               break;

            case (char)0xAE:
               _spltBtnGlryUL.SelectedItem = (uint)UnorderedListGalleryButton.Arrow;
               break;

            case null:
               _spltBtnGlryUL.SelectedItem = uint.MaxValue;
               break;
         }
      }

      void InputFormat_BoxFrameChanged(object sender, EventArgs e)
      {
         var btn = _tglBtnsBorder[(int)BorderButton.Outside];
         btn.BooleanValue = _textControl.InputFormat.BoxFrame;
      }

      void InputFormat_BottomFrameLineChanged(object sender, EventArgs e)
      {
         var btn = _tglBtnsBorder[(int)BorderButton.Bottom];
         btn.BooleanValue = _textControl.InputFormat.BottomFrameLine;
      }

      void InputFormat_AllFrameLinesChanged(object sender, EventArgs e)
      {
         var btn = _tglBtnsBorder[(int)BorderButton.All];
         btn.BooleanValue = _textControl.InputFormat.AllFrameLines;
      }

      void InputFormat_StyleNameChanged(object sender, EventArgs e)
      {
         SelectCurrentStyleItem();
      }

      void InputFormat_UnderlineChanged(object sender, EventArgs e)
      {
         _fntCntrlState.Underline = FontUnderlineFromNullableBool(_textControl.InputFormat.Underline);
         _fntCntrl.Underline = _fntCntrlState.Underline;
      }

      void InputFormat_TextColorChanged(object sender, EventArgs e)
      {
         _fntCntrlState.ForegroundColor = _textControl.InputFormat.TextColor ?? Color.FromKnownColor(KnownColor.WindowText);
         _fntCntrl.ForegroundColor = _fntCntrlState.ForegroundColor;
      }

      void InputFormat_TextBackColorChanged(object sender, EventArgs e)
      {
         _fntCntrlState.BackgroundColor = _textControl.InputFormat.TextBackColor ?? Color.FromKnownColor(KnownColor.Window);
         _fntCntrl.BackgroundColor = _fntCntrlState.BackgroundColor;
      }

      void InputFormat_SuperscriptChanged(object sender, EventArgs e)
      {
         if ((_textControl.InputFormat.Superscript != null) && (_textControl.InputFormat.Superscript == true))
         {
            _fntCntrlState.VerticalPositioning = FontVerticalPosition.SuperScript;
         }
         else _fntCntrlState.VerticalPositioning = FontVerticalPosition.NotSet;

         _fntCntrl.VerticalPositioning = _fntCntrlState.VerticalPositioning;
      }

      void InputFormat_SubscriptChanged(object sender, EventArgs e)
      {
         if ((_textControl.InputFormat.Subscript != null) && (_textControl.InputFormat.Subscript == true))
         {
            _fntCntrlState.VerticalPositioning = FontVerticalPosition.SubScript;
         }
         else _fntCntrlState.VerticalPositioning = FontVerticalPosition.NotSet;

         _fntCntrl.VerticalPositioning = _fntCntrlState.VerticalPositioning;
      }

      void InputFormat_StrikeoutChanged(object sender, EventArgs e)
      {
         _fntCntrlState.Strikethrough = FontPropsFromNullableBool(_textControl.InputFormat.Strikeout);
         _fntCntrl.Strikethrough = _fntCntrlState.Strikethrough;
      }

      void InputFormat_ItalicChanged(object sender, EventArgs e)
      {
         _fntCntrlState.Italic = FontPropsFromNullableBool(_textControl.InputFormat.Italic);
         _fntCntrl.Italic = _fntCntrlState.Italic;
      }

      void InputFormat_FontSizeChanged(object sender, EventArgs e)
      {
         if (_bInFontCnrlPreviewEvent) return;

         _fntCntrlState.Size = (_textControl.InputFormat.FontSize != null) ? (decimal)_textControl.InputFormat.FontSize / 20 : 0;
         _fntCntrl.Size = _fntCntrlState.Size;
      }

      void InputFormat_FontFamilyChanged(object sender, EventArgs e)
      {
         if (_bInFontCnrlPreviewEvent) return;

         _fntCntrlState.Family = _textControl.InputFormat.FontFamily ?? string.Empty;
         _fntCntrl.Family = _fntCntrlState.Family;
      }

      void InputFormat_BoldChanged(object sender, EventArgs e)
      {
         _fntCntrlState.Bold = FontPropsFromNullableBool(_textControl.InputFormat.Bold);
         _fntCntrl.Bold = _fntCntrlState.Bold;
      }

      #endregion // Text Control event handlers

      #region "  Ribbon event handlers  "

      void TglBtnsBorder_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         var btn = sender as RibbonToggleButton;
         if (btn == null) return;

         var cmd = (RibbonMarkupCommand_TabHome)btn.CommandID;

         switch (cmd)
         {
            case RibbonMarkupCommand_TabHome.cmdTglBtnBorderLeft:
               BorderButtonPressed(BorderButton.Left, btn.BooleanValue);
               break;

            case RibbonMarkupCommand_TabHome.cmdTglBtnBorderTop:
               BorderButtonPressed(BorderButton.Top, btn.BooleanValue);
               break;

            case RibbonMarkupCommand_TabHome.cmdTglBtnBorderRight:
               BorderButtonPressed(BorderButton.Right, btn.BooleanValue);
               break;

            case RibbonMarkupCommand_TabHome.cmdTglBtnBorderBottom:
               BorderButtonPressed(BorderButton.Bottom, btn.BooleanValue);
               break;

            case RibbonMarkupCommand_TabHome.cmdTglBtnBorderOutside:
               BorderButtonPressed(BorderButton.Outside, btn.BooleanValue);
               break;

            case RibbonMarkupCommand_TabHome.cmdTglBtnBorderAll:
               BorderButtonPressed(BorderButton.All, btn.BooleanValue);
               break;

            case RibbonMarkupCommand_TabHome.cmdTglBtnBorderInsideHor:
               BorderButtonPressed(BorderButton.InsideHor, btn.BooleanValue);
               break;

            case RibbonMarkupCommand_TabHome.cmdTglBtnBorderInsideVert:
               BorderButtonPressed(BorderButton.InsideVert, btn.BooleanValue);
               break;
         }
      }

      private void BorderButtonPressed(BorderButton borderButton, bool toggled)
      {
         switch (borderButton)
         {
            case BorderButton.Left:
               _textControl.InputFormat.LeftFrameLine = toggled;
               break;

            case BorderButton.Top:
               _textControl.InputFormat.TopFrameLine = toggled;
               break;

            case BorderButton.Right:
               _textControl.InputFormat.RightFrameLine = toggled;
               break;

            case BorderButton.Bottom:
               _textControl.InputFormat.BottomFrameLine = toggled;
               break;

            case BorderButton.Outside:
               _textControl.InputFormat.BoxFrame = toggled;
               break;

            case BorderButton.All:
               _textControl.InputFormat.AllFrameLines = toggled;
               break;

            case BorderButton.InsideHor:
               _textControl.InputFormat.InnerHorizontalFrameLines = toggled;
               break;

            case BorderButton.InsideVert:
               _textControl.InputFormat.InnerVerticalFrameLines = toggled;
               break;
         }
      }

      void TglBtnsLineSpace_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         var chk = sender as RibbonCheckBox;
         if (chk == null) return;

         var cmd = (RibbonMarkupCommand_TabHome)chk.CommandID;

         if (chk.BooleanValue)
         {
            UntoggleCheckboxes(_chkBoxesLineSpace, chk);
            switch (cmd)
            {
               case RibbonMarkupCommand_TabHome.cmdChkBoxLnSpace_1_00:
                  LineSpaceButtonPressed(LineSpaceChkBox._1_00);
                  break;

               case RibbonMarkupCommand_TabHome.cmdChkBoxLnSpace_1_15:
                  LineSpaceButtonPressed(LineSpaceChkBox._1_15);
                  break;

               case RibbonMarkupCommand_TabHome.cmdChkBoxLnSpace_1_50:
                  LineSpaceButtonPressed(LineSpaceChkBox._1_50);
                  break;

               case RibbonMarkupCommand_TabHome.cmdChkBoxLnSpace_2_00:
                  LineSpaceButtonPressed(LineSpaceChkBox._2_00);
                  break;

               case RibbonMarkupCommand_TabHome.cmdChkBoxLnSpace_2_50:
                  LineSpaceButtonPressed(LineSpaceChkBox._2_50);
                  break;

               case RibbonMarkupCommand_TabHome.cmdChkBoxLnSpace_3_00:
                  LineSpaceButtonPressed(LineSpaceChkBox._3_00);
                  break;
            }
         }
         else chk.BooleanValue = true;
      }

      private void LineSpaceButtonPressed(LineSpaceChkBox lineSpaceBtn)
      {
         switch (lineSpaceBtn)
         {  
            case LineSpaceChkBox._1_00:
               _textControl.InputFormat.LineSpacing = 100;
               break;

            case LineSpaceChkBox._1_15:
               _textControl.InputFormat.LineSpacing = 115;
               break;

            case LineSpaceChkBox._1_50:
               _textControl.InputFormat.LineSpacing = 150;
               break;

            case LineSpaceChkBox._2_00:
               _textControl.InputFormat.LineSpacing = 200;
               break;

            case LineSpaceChkBox._2_50:
               _textControl.InputFormat.LineSpacing = 250;
               break;

            case LineSpaceChkBox._3_00:
               _textControl.InputFormat.LineSpacing = 300;
               break;
         }
      }

      void TglBtnsHorAlign_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         var btn = sender as RibbonToggleButton;
         if (btn == null) return;

         if (btn.BooleanValue)
         {
            UntoggleToggleButtons(_tglBtnsHorAlign, btn);

            var cmd = (RibbonMarkupCommand_TabHome)btn.CommandID;

            switch (cmd)
            {
               case RibbonMarkupCommand_TabHome.cmdTglBtnHorAlignLeft:
                  HorAlignButtonPressed(HorAlignButton.Left);
                  break;

               case RibbonMarkupCommand_TabHome.cmdTglBtnHorAlignCenter:
                  HorAlignButtonPressed(HorAlignButton.Center);
                  break;

               case RibbonMarkupCommand_TabHome.cmdTglBtnHorAlignRight:
                  HorAlignButtonPressed(HorAlignButton.Right);
                  break;

               case RibbonMarkupCommand_TabHome.cmdTglBtnHorAlignJust:
                  HorAlignButtonPressed(HorAlignButton.Justify);
                  break;
            }
         }
         else btn.BooleanValue = true;
      }

      private void HorAlignButtonPressed(HorAlignButton horAlignButton)
      {
         switch (horAlignButton)
         {
            case HorAlignButton.Left:
               _textControl.InputFormat.LeftAligned = true;
               break;

            case HorAlignButton.Center:
               _textControl.InputFormat.Centered = true;
               break;

            case HorAlignButton.Right:
               _textControl.InputFormat.RightAligned = true;
               break;

            case HorAlignButton.Justify:
               _textControl.InputFormat.Justified = true;
               break;
         }
      }

      void SplitBtnGalleryUL_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (e.Key == null)    // Split button pressed
         {
            _textControl.InputFormat.BulletedList = !_spltBtnGlryUL.BooleanValue;
         }
         else if (e.Key.PropertyKey == RibbonProperties.SelectedItem)
         {
            var rsbg = sender as RibbonSplitButtonGallery;
            if (rsbg == null) return;

            var btnID = (UnorderedListGalleryButton)rsbg.SelectedItem;
            SetBulletChar(btnID);
         }
      }

      private void SetBulletChar(UnorderedListGalleryButton btnID)
      {
         switch (btnID)
         {
            case UnorderedListGalleryButton.Arrow:
               _textControl.InputFormat.BulletCharacter = (char)0xAE;
               break;

            case UnorderedListGalleryButton.Asterisk:
               _textControl.InputFormat.BulletCharacter = (char)0x2A;
               break;

            case UnorderedListGalleryButton.Bullet:
               _textControl.InputFormat.BulletCharacter = (char)0xB7;
               break;

            case UnorderedListGalleryButton.Dash:
               _textControl.InputFormat.BulletCharacter = (char)0x2D;
               break;

            case UnorderedListGalleryButton.Diamond:
               _textControl.InputFormat.BulletCharacter = (char)0xA8;
               break;
         }
      }

      void SplitBtnGalleryUL_CategoriesReady(object sender, EventArgs e)
      {
         var categories = _spltBtnGlryUL.Categories;
         categories.Clear();
         categories.Add(new GalleryItemPropertySet() { Label = "Bullet Characters", CategoryID = 1 });
      }

      void SplitBtnGalleryUL_ItemsSourceReady(object sender, EventArgs e)
      {
         // Set _splitBtnGalleryUL items

         var itemsSource = _spltBtnGlryUL.ItemsSource;
         itemsSource.Clear();

         foreach (Image img in _imgLstGalleryUL.Images)
         {
            itemsSource.Add(new GalleryItemPropertySet()
            {
               ItemImage = _ribbonMain.ConvertToUIImage((Bitmap)img),
               CategoryID = 1,               
            });
         }
      }

      void ListFmtDialogHandler(object sender, ExecuteEventArgs e)
      {
         if (_textControl.ListFormatDialog() == System.Windows.Forms.DialogResult.OK)
         {
            _fileHandler.DocumentDirty = true;
         }
      }

      void SplitBtnGalleryOL_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (e.Key == null)
         {
            _textControl.InputFormat.NumberedList = !_spltBtnGlryOL.BooleanValue;
         }
         else if (e.Key.PropertyKey == RibbonProperties.SelectedItem)
         {
            var rsbg = sender as RibbonSplitButtonGallery;
            if (rsbg == null) return;

            var btnID = (OrderedListGalleryButton)rsbg.SelectedItem;
            SetNumberedListFormat(btnID);
         }
      }

      private void SetNumberedListFormat(OrderedListGalleryButton btnID)
      {
         switch (btnID)
         {
            case OrderedListGalleryButton._123:
               _textControl.InputFormat.NumberedListFormat = TXTextControl.NumberFormat.ArabicNumbers;
               break;

            case OrderedListGalleryButton.ABC:
               _textControl.InputFormat.NumberedListFormat = TXTextControl.NumberFormat.CapitalLetters;
               break;

            case OrderedListGalleryButton.abc:
               _textControl.InputFormat.NumberedListFormat = TXTextControl.NumberFormat.Letters;
               break;

            case OrderedListGalleryButton.I_II_III:
               _textControl.InputFormat.NumberedListFormat = TXTextControl.NumberFormat.RomanNumbers;
               break;

            case OrderedListGalleryButton.i_ii_iii:
               _textControl.InputFormat.NumberedListFormat = TXTextControl.NumberFormat.SmallRomanNumbers;
               break;
         }
      }

      void SplitBtnGalleryOL_CategoriesReady(object sender, EventArgs e)
      {
         var categories = _spltBtnGlryOL.Categories;
         categories.Clear();
         categories.Add(new GalleryItemPropertySet() { Label = "Number Format", CategoryID = 1 });
      }

      void SplitBtnGalleryOL_ItemsSourceReady(object sender, EventArgs e)
      {
         // Set _splitBtnGalleryOL items

         var itemsSource = _spltBtnGlryOL.ItemsSource;
         itemsSource.Clear();

         foreach (var img in _imgLstGalleryOL.Images)
         {
            itemsSource.Add(new GalleryItemPropertySet()
            {
               ItemImage = _ribbonMain.ConvertToUIImage((Bitmap)img),
               CategoryID = 1,
            });
         }
      }

      void SplitBtnGalleryMultilvl_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (e.Key == null)
         {
            _textControl.InputFormat.StructuredList = !_spltBtnGlryMultilvl.BooleanValue;
         }
         else if (e.Key.PropertyKey == RibbonProperties.SelectedItem)
         {
            var rsbg = sender as RibbonSplitButtonGallery;
            if (rsbg == null) return;

            var btnID = (MultiLvlListGalleryButton)rsbg.SelectedItem;
            SetStructuredListFormat(btnID);
         }
      }

      private void SetStructuredListFormat(MultiLvlListGalleryButton btnID)
      {
         switch (btnID)
         {
            case MultiLvlListGalleryButton._123:
               _textControl.InputFormat.StructuredListFormat = TXTextControl.NumberFormat.ArabicNumbers;
               break;

            case MultiLvlListGalleryButton.ABC:
               _textControl.InputFormat.StructuredListFormat = TXTextControl.NumberFormat.CapitalLetters;
               break;

            case MultiLvlListGalleryButton.abc:
               _textControl.InputFormat.StructuredListFormat = TXTextControl.NumberFormat.Letters;
               break;

            case MultiLvlListGalleryButton.I_II_III:
               _textControl.InputFormat.StructuredListFormat = TXTextControl.NumberFormat.RomanNumbers;
               break;

            case MultiLvlListGalleryButton.i_ii_iii:
               _textControl.InputFormat.StructuredListFormat = TXTextControl.NumberFormat.SmallRomanNumbers;
               break;
         }
      }

      void SplitBtnGalleryMultilvl_CategoriesReady(object sender, EventArgs e)
      {
         var categories = _spltBtnGlryMultilvl.Categories;
         categories.Clear();
         categories.Add(new GalleryItemPropertySet() { Label = "Number Format", CategoryID = 1 });
      }

      void SplitBtnGalleryMultilvl_ItemsSourceReady(object sender, EventArgs e)
      {
         // Set _splitBtnGalleryUL items

         var itemsSource = _spltBtnGlryMultilvl.ItemsSource;
         itemsSource.Clear();

         foreach (Image img in _imgLstGalleryMultiLvl.Images)
         {
            itemsSource.Add(new GalleryItemPropertySet()
            {
               ItemImage = _ribbonMain.ConvertToUIImage((Bitmap)img),
               CategoryID = 1,
            });
         }
      }

      void BtnPaste_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Paste();
      }

      void BtnCopy_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Copy();
      }

      void BtnCut_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Cut();
      }

      void BtnPasteSpecial_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         var frm = new FrmPasteSpecial() { TextControl = _textControl };

         if ((frm.ShowDialog() == System.Windows.Forms.DialogResult.OK) && (frm.SelectedItem != null))
         {
            _textControl.Paste((TXTextControl.ClipboardFormat)frm.SelectedItem);
         }
      }

      void BtnIndIncr_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Selection.IncreaseIndent();
      }

      void BtnIndDecr_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Selection.DecreaseIndent();
      }

      void BtnTabPos_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (_textControl.TabDialog() == System.Windows.Forms.DialogResult.OK)
         {
            _fileHandler.DocumentDirty = true;
         }
      }

      void TglBtnCntrlChar_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         var btn = sender as RibbonToggleButton;
         if (btn == null) return;

         _textControl.ControlChars = btn.BooleanValue;
      }

      void BtnParFormat_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (_textControl.ParagraphFormatDialog() == System.Windows.Forms.DialogResult.OK)
         {
            _fileHandler.DocumentDirty = true;
         }
      }

      void BtnManageStyles_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (_textControl.FormattingStylesDialog() == System.Windows.Forms.DialogResult.OK)
         {
            _fileHandler.DocumentDirty = true;
         }
      }

      void SpltBtnGlryStyles_CategoriesReady(object sender, EventArgs e)
      {
         var categories = _spltBtnGlryStyles.Categories;
         categories.Clear();
         categories.Add(new GalleryItemPropertySet() { Label = "Formatting Styles", CategoryID = 1 });
      }

      void SpltBtnGlryStyles_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (e.Key == null)   // Split button pressed
         {
            BtnManageStyles_ExecuteEvent(sender, e);
         }
         else if (e.Key.PropertyKey == RibbonProperties.SelectedItem)
         {
            if (_spltBtnGlryStyles.SelectedItem != uint.MaxValue)
            {
               _textControl.InputFormat.StyleName = _textControl.InputFormat.StyleNames[_spltBtnGlryStyles.SelectedItem];
            }
         }
      }

      void FntCntrl_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         var dif = _fntCntrlState.CompareTo(_fntCntrl);
         if (dif == FontControlState.Difference.None) return;

         foreach (FontControlState.Difference d in Enum.GetValues(typeof(FontControlState.Difference)))
         {
            switch (dif & d)
            {
               case FontControlState.Difference.Family:
                  _textControl.InputFormat.FontFamily = _fntCntrl.Family;
                  break;

               case FontControlState.Difference.Size:
                  _textControl.InputFormat.FontSize = (int)(20 * _fntCntrl.Size);
                  break;

               case FontControlState.Difference.Bold:
                  _textControl.InputFormat.Bold = _fntCntrl.Bold == FontProperties.Set;
                  break;

               case FontControlState.Difference.Italic:
                  _textControl.InputFormat.Italic = _fntCntrl.Italic == FontProperties.Set;
                  break;

               case FontControlState.Difference.Underline:
                  _textControl.InputFormat.Underline = _fntCntrl.Underline == FontUnderline.Set;
                  break;

               case FontControlState.Difference.Strikethrough:
                  _textControl.InputFormat.Strikeout = _fntCntrl.Strikethrough == FontProperties.Set;
                  break;

               case FontControlState.Difference.VerticalPositioning:
                  switch (_fntCntrl.VerticalPositioning)
                  {
                     case FontVerticalPosition.NotSet:
                        _textControl.InputFormat.Subscript = _textControl.InputFormat.Superscript = false;
                        break;

                     case FontVerticalPosition.SubScript:
                        _textControl.InputFormat.Superscript = false;
                        _textControl.InputFormat.Subscript = true;
                        break;

                     case FontVerticalPosition.SuperScript:
                        _textControl.InputFormat.Subscript = false;
                        _textControl.InputFormat.Superscript = true;
                        break;
                  }
                  break;

               case FontControlState.Difference.BackgroundColor:
                  _textControl.InputFormat.TextBackColor = _fntCntrl.BackgroundColor;
                  break;

               case FontControlState.Difference.ForegroundColor:
                  _textControl.InputFormat.TextColor = _fntCntrl.ForegroundColor;
                  break;
            }
         }

         // Store current font control state
         _fntCntrlState.CopyValues(_fntCntrl);
      }

      void FntCntrl_CancelPreviewEvent(object sender, ExecuteEventArgs e)
      {
         var fontProps = (IPropertyStore)e.CurrentValue.PropVariant.Value;
         UpdateFontFamilyAndSize(fontProps);
      }

      void FntCntrl_PreviewEvent(object sender, ExecuteEventArgs e)
      {
         PropVariant props;

         _bInFontCnrlPreviewEvent = true;

         e.CommandExecutionProperties.GetValue(
            ref RibbonProperties.FontProperties_ChangedProperties,
            out props);

         var propStore = (IPropertyStore)props.Value;
         UpdateFontFamilyAndSize(propStore);

         _bInFontCnrlPreviewEvent = false;
      }

      void BtnGoTo_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         TXTextControl.PageCollection pages = null;

         // Find out if installed textcontrol version supports 
         // GetPages() (and therefore also sections):
         try
         {
            pages = _textControl.GetPages();
         }
         catch (Exception ex)
         {
            MessageBox.Show(ex.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
            return;
         }

         var gotoDialog = new FrmGoTo();
         gotoDialog.MaxPages = _textControl.Pages;
         gotoDialog.MaxSections = _textControl.Sections.Count;

         if (gotoDialog.ShowDialog() != System.Windows.Forms.DialogResult.OK) return;

         if (gotoDialog.IsSection)
         {
            _textControl.Selection.Start = _textControl.Sections[gotoDialog.Number].Start - 1;
            _textControl.ScrollLocation = (_textControl.Sections.GetItem().Start <= _textControl.TextChars.Count) ?
                new Point(_textControl.ScrollLocation.X, _textControl.TextChars[_textControl.Sections.GetItem().Start].Bounds.Top) :
                new Point(_textControl.ScrollLocation.X, _textControl.Lines[_textControl.Lines.Count].TextBounds.Top);
         }
         else
         {
            _textControl.Selection.Start = pages[gotoDialog.Number].Start - 1;
            _textControl.ScrollLocation = new Point(_textControl.ScrollLocation.X, pages.GetItem().Bounds.Top);
         }
      }

      void BtnReplace_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Replace();
      }

      void BtnFind_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         _textControl.Find();
      }

      #endregion // Event handlers

      #region "  Helpers  "

      private void CreateRibbonItems_TabHome()
      {
         _tabHome = new RibbonTab(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdTabHome);
         _grpClipboard = new RibbonGroup(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdGrpClipboard);
         _btnPaste = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnPaste);
         _btnCut = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnCut);
         _btnCopy = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnCopy);
         _btnPasteSpecial = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnPasteSpecial);
         _btnFmtBullets = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnFmtBullets);
         _btnFmtNumbering = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnFmtNumbering);
         _btnFmtStructured = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnFmtMultiLvl);
         _spltBtnGlryUL = new RibbonSplitButtonGallery(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnLstUnordered);
         _spltBtnGlryOL = new RibbonSplitButtonGallery(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnLstOrdered);
         _spltBtnGlryMultilvl = new RibbonSplitButtonGallery(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnLstMultiLvl);
         _btnIndDecr = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnIndDecr);
         _btnIndIncr = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnIndIncr);
         _btnTabPos = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnTabPos);
         _tglBtnCntrlChar = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdTglBtnCntrlChar);
         _tglBtnsHorAlign = new RibbonToggleButton[Enum.GetValues(typeof(HorAlignButton)).Length];
         _tglBtnsHorAlign[(int)HorAlignButton.Left] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdTglBtnHorAlignLeft);
         _tglBtnsHorAlign[(int)HorAlignButton.Center] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdTglBtnHorAlignCenter);
         _tglBtnsHorAlign[(int)HorAlignButton.Right] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdTglBtnHorAlignRight);
         _tglBtnsHorAlign[(int)HorAlignButton.Justify] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdTglBtnHorAlignJust);
         _chkBoxesLineSpace = new RibbonCheckBox[Enum.GetValues(typeof(LineSpaceChkBox)).Length];
         _chkBoxesLineSpace[(int)LineSpaceChkBox._1_00] = new RibbonCheckBox(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdChkBoxLnSpace_1_00);
         _chkBoxesLineSpace[(int)LineSpaceChkBox._1_15] = new RibbonCheckBox(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdChkBoxLnSpace_1_15);
         _chkBoxesLineSpace[(int)LineSpaceChkBox._1_50] = new RibbonCheckBox(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdChkBoxLnSpace_1_50);
         _chkBoxesLineSpace[(int)LineSpaceChkBox._2_00] = new RibbonCheckBox(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdChkBoxLnSpace_2_00);
         _chkBoxesLineSpace[(int)LineSpaceChkBox._2_50] = new RibbonCheckBox(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdChkBoxLnSpace_2_50);
         _chkBoxesLineSpace[(int)LineSpaceChkBox._3_00] = new RibbonCheckBox(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdChkBoxLnSpace_3_00);
         _tglBtnsBorder = new RibbonToggleButton[Enum.GetValues(typeof(BorderButton)).Length];
         _tglBtnsBorder[(int)BorderButton.Left] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdTglBtnBorderLeft);
         _tglBtnsBorder[(int)BorderButton.Top] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdTglBtnBorderTop);
         _tglBtnsBorder[(int)BorderButton.Right] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdTglBtnBorderRight);
         _tglBtnsBorder[(int)BorderButton.Bottom] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdTglBtnBorderBottom);
         _tglBtnsBorder[(int)BorderButton.Outside] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdTglBtnBorderOutside);
         _tglBtnsBorder[(int)BorderButton.All] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdTglBtnBorderAll);
         _tglBtnsBorder[(int)BorderButton.InsideHor] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdTglBtnBorderInsideHor);
         _tglBtnsBorder[(int)BorderButton.InsideVert] = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdTglBtnBorderInsideVert);
         _btnParFormat = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnParFormat);
         _spltBtnGlryStyles = new RibbonSplitButtonGallery(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdSpltBtnGlryStyles);
         _btnManageStyles = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnManageStyles);
         _fntCntrl = new RibbonFontControl(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdFntCntrol);
         _btnFind = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnFind);
         _btnReplace = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnReplace);
         _btnGoTo = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabHome.cmdBtnGoTo);
      }

      private void AttachRibbonEvents_TabHome()
      {
         _btnPaste.ExecuteEvent += BtnPaste_ExecuteEvent;
         _btnCut.ExecuteEvent += BtnCut_ExecuteEvent;
         _btnCopy.ExecuteEvent += BtnCopy_ExecuteEvent;
         _btnPasteSpecial.ExecuteEvent += BtnPasteSpecial_ExecuteEvent;
         _spltBtnGlryUL.CategoriesReady += SplitBtnGalleryUL_CategoriesReady;
         _spltBtnGlryUL.ItemsSourceReady += SplitBtnGalleryUL_ItemsSourceReady;
         _spltBtnGlryUL.ExecuteEvent += SplitBtnGalleryUL_ExecuteEvent;
         _btnFmtBullets.ExecuteEvent += ListFmtDialogHandler;
         _spltBtnGlryOL.CategoriesReady += SplitBtnGalleryOL_CategoriesReady;
         _spltBtnGlryOL.ItemsSourceReady += SplitBtnGalleryOL_ItemsSourceReady;
         _spltBtnGlryOL.ExecuteEvent += SplitBtnGalleryOL_ExecuteEvent;
         _btnFmtNumbering.ExecuteEvent += ListFmtDialogHandler;
         _spltBtnGlryMultilvl.ExecuteEvent += SplitBtnGalleryMultilvl_ExecuteEvent;
         _spltBtnGlryMultilvl.ItemsSourceReady += SplitBtnGalleryMultilvl_ItemsSourceReady;
         _spltBtnGlryMultilvl.CategoriesReady += SplitBtnGalleryMultilvl_CategoriesReady;
         _btnFmtStructured.ExecuteEvent += ListFmtDialogHandler;
         _btnIndDecr.ExecuteEvent += BtnIndDecr_ExecuteEvent;
         _btnIndIncr.ExecuteEvent += BtnIndIncr_ExecuteEvent;
         _btnTabPos.ExecuteEvent += BtnTabPos_ExecuteEvent;
         _tglBtnCntrlChar.ExecuteEvent += TglBtnCntrlChar_ExecuteEvent;

         foreach (var btn in _tglBtnsHorAlign)
         {
            btn.ExecuteEvent += TglBtnsHorAlign_ExecuteEvent;
         }

         foreach (var chk in _chkBoxesLineSpace)
         {
            chk.ExecuteEvent += TglBtnsLineSpace_ExecuteEvent;
         }

         foreach (var btn in _tglBtnsBorder)
         {
            btn.ExecuteEvent += TglBtnsBorder_ExecuteEvent;
         }

         _btnParFormat.ExecuteEvent += BtnParFormat_ExecuteEvent;
         _spltBtnGlryStyles.ExecuteEvent += SpltBtnGlryStyles_ExecuteEvent;
         _spltBtnGlryStyles.CategoriesReady += SpltBtnGlryStyles_CategoriesReady;
         _btnManageStyles.ExecuteEvent += BtnManageStyles_ExecuteEvent;
         _fntCntrl.ExecuteEvent += FntCntrl_ExecuteEvent;
         _fntCntrl.PreviewEvent += FntCntrl_PreviewEvent;
         _fntCntrl.CancelPreviewEvent += FntCntrl_CancelPreviewEvent;

         _btnFind.ExecuteEvent += BtnFind_ExecuteEvent;
         _btnReplace.ExecuteEvent += BtnReplace_ExecuteEvent;
         _btnGoTo.ExecuteEvent += BtnGoTo_ExecuteEvent;
      }

      private FontProperties FontPropsFromNullableBool(bool? value)
      {
         if (value == null) return FontProperties.NotAvailable;
         if (value == true) return FontProperties.Set;
         else return FontProperties.NotSet;
      }

      private FontUnderline FontUnderlineFromNullableBool(bool? value)
      {
         if (value == null) return FontUnderline.NotAvailable;
         if (value == true) return FontUnderline.Set;
         else return FontUnderline.NotSet;
      }

      private void UpdateCopyPaste()
      {
         _btnCut.Enabled = _textControl.CanCopy;
         _btnCopy.Enabled = _textControl.CanCopy;
         _btnPaste.Enabled = _textControl.CanPaste;
         _btnPasteSpecial.Enabled = _textControl.CanPaste;
      }

      private void UpdateUndoRedo()
      {
         _btnUndo.Enabled = _textControl.CanUndo;
         _btnRedo.Enabled = _textControl.CanRedo;
      }

      private void InitInputFormatDependentControls()
      {
         InitFontControl();
         InitParFormatGroup();
      }

      private void InitParFormatGroup()
      {
         var inpFmt = _textControl.InputFormat;

         if (inpFmt.LeftAligned == true) _tglBtnsHorAlign[(int)HorAlignButton.Left].BooleanValue = true;
         else if (inpFmt.Centered == true) _tglBtnsHorAlign[(int)HorAlignButton.Center].BooleanValue = true;
         else if (inpFmt.RightAligned == true) _tglBtnsHorAlign[(int)HorAlignButton.Right].BooleanValue = true;
         else if (inpFmt.Justified == true) _tglBtnsHorAlign[(int)HorAlignButton.Justify].BooleanValue = true;

         switch (_textControl.InputFormat.LineSpacing)
         {
            case 100:
               _chkBoxesLineSpace[(int)LineSpaceChkBox._1_00].BooleanValue = true;
               break;

            case 115:
               _chkBoxesLineSpace[(int)LineSpaceChkBox._1_15].BooleanValue = true;
               break;

            case 150:
               _chkBoxesLineSpace[(int)LineSpaceChkBox._1_50].BooleanValue = true;
               break;

            case 200:
               _chkBoxesLineSpace[(int)LineSpaceChkBox._2_00].BooleanValue = true;
               break;

            case 250:
               _chkBoxesLineSpace[(int)LineSpaceChkBox._2_50].BooleanValue = true;
               break;

            case 300:
               _chkBoxesLineSpace[(int)LineSpaceChkBox._3_00].BooleanValue = true;
               break;
         }
      }

      private void InitFontControl()
      {
         _fntCntrl.BackgroundColor = _textControl.InputFormat.TextBackColor ?? Color.FromKnownColor(KnownColor.Window);
         _fntCntrl.Bold = FontPropsFromNullableBool(_textControl.InputFormat.Bold);
         _fntCntrl.Family = _textControl.InputFormat.FontFamily ?? string.Empty;
         _fntCntrl.ForegroundColor = _textControl.InputFormat.TextColor ?? Color.FromKnownColor(KnownColor.WindowText);
         _fntCntrl.Italic = FontPropsFromNullableBool(_textControl.InputFormat.Italic);
         _fntCntrl.Size = (_textControl.InputFormat.FontSize != null) ? (decimal)_textControl.InputFormat.FontSize / 20 : 0;
         _fntCntrl.Strikethrough = FontPropsFromNullableBool(_textControl.InputFormat.Strikeout);
         _fntCntrl.Underline = FontUnderlineFromNullableBool(_textControl.InputFormat.Underline);

         if ((_textControl.InputFormat.Superscript != null) && (_textControl.InputFormat.Superscript == true))
         {
            _fntCntrl.VerticalPositioning = FontVerticalPosition.SuperScript;
         }
         else if ((_textControl.InputFormat.Subscript != null) && (_textControl.InputFormat.Subscript == true))
         {
            _fntCntrl.VerticalPositioning = FontVerticalPosition.SubScript;
         }
         else _fntCntrl.VerticalPositioning = FontVerticalPosition.NotSet;

         _fntCntrlState.CopyValues(_fntCntrl);
      }

      private void SelectCurrentStyleItem()
      {
         var styles = new List<string>(_textControl.InputFormat.StyleNames);
         _spltBtnGlryStyles.SelectedItem = (uint)styles.FindIndex(str => str == _textControl.InputFormat.StyleName);
      }

      private void UpdateFontFamilyAndSize(IPropertyStore propStore)
      {
         PropVariant propValue;

         var fontProps = new FontPropertyStore(propStore);

         if (propStore.GetValue(ref RibbonProperties.FontProperties_Family, out propValue) == HRESULT.S_OK)
         {
            _textControl.InputFormat.FontFamily = fontProps.Family;
         }
         if (propStore.GetValue(ref RibbonProperties.FontProperties_Size, out propValue) == HRESULT.S_OK)
         {
            _textControl.InputFormat.FontSize = (int)(fontProps.Size * 20);
         }
      }

      #endregion // Helpers
   }
}
