﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				Tab_Insert.cs
** description:	This file contains event handlers, fields and helper methods concerning the “Insert” 
**                ribbon tab.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using RibbonLib;
using RibbonLib.Controls;
using RibbonLib.Controls.Events;
using RibbonLib.Interop;
using TXTextControl;


namespace TX_Text_Control_Words
{
   public partial class FormMain : Form
   {
      #region "  Private fields  "

      private RibbonButton _btnInsertTable;
      private RibbonDropDownGallery _drpDnGlrySymbol;
      private RibbonButton[] _btnsGallerySymbol;
      private RibbonButton _btnBlankPage;
      private RibbonButton _btnPageBreak;
      private RibbonButton _btnInsertImage;
      private RibbonButton _btnInsertHyperlink;
      private RibbonButton _btnEditHyperlink;
      private RibbonButton _btnInsertBookmark;
      private RibbonButton _btnEditBookmark;
      private RibbonButton _btnDeleteBookmark;
      private RibbonToggleButton _tglBtnShowBookmarkMarkers;
      private RibbonButton _btnEditHeader;
      private RibbonButton _btnRemoveHeader;
      private RibbonButton _btnEditFooter;
      private RibbonButton _btnRemoveFooter;
      private RibbonButton _btnInsertTextFrame;
      private RibbonToggleButton _tglBtnShowTextFrameMarkerLines;
      private RibbonButton _btnInsertFile;
      private RibbonButton _btnPgNumTopOfPage;
      private RibbonButton _btnPgNumBottomOfPage;
      private RibbonButton _btnPgNumRemove;

      /// <summary>
      /// Characters in symbol drop down gallery in same order as enum SymbolButtonTextIndex.
      /// </summary>
      private static string[] SymbolButtonText = 
      {
         "™", "ß", "€", "£", "¥", "©", "®", "±", "≠", "≤", "≥", "÷", "×", "∞", "µ", "α", "π", "Ω", "∑", "☺",
      };

      #endregion // Private fields

      #region "  Enumerations  "

      /// <summary>
      /// Character indexes in SymbolButtonText.
      /// </summary>
      private enum SymbolButtonTextIndex
      {
         TM,
         SzLig,
         Euro,
         Pound,
         Yen,
         Copyright,
         Registered,
         PlusMinus,
         Inequal,
         Lte,
         Gte,
         Division,
         Multiplication,
         Infinity,
         Micro,
         Alpha,
         Pi,
         Omega,
         Sigma,
         Smiling,
      }

      enum RibbonMarkupCommand_TabInsert : uint
      {
         cmdTabInsert = 2000,
         cmdBtnBlankPage = 2020,
         cmdBtnPageBreak = 2021,
         cmdBtnInsertTable = 2022,
         cmdBtnInsertImage = 2023,
         cmdDrpDnBtnHyperlink = 2024,
         cmdBtnInsertHyperlink = 2025,
         cmdBtnEditHyperlink = 2026,
         cmdDrpDnBtnBookmark = 2027,
         cmdBtnInsertBookmark = 2028,
         cmdBtnEditBookmark = 2029,
         cmdTglBtnShowBookmarkMarkers = 2030,
         cmdDrpDnBtnHeader = 2031,
         cmdBtnEditHeader = 2032,
         cmdBtnRemoveHeader = 2033,
         cmdDrpDnBtnFooter = 2034,
         cmdBtnEditFooter = 2035,
         cmdBtnRemoveFooter = 2036,
         cmdDrpDnBtnTextFrame = 2037,
         cmdBtnInsertTextFrame = 2038,
         cmdTglBtnShowTextFrameMarkerLines = 2039,
         cmdBtnInsertFile = 2040,
         cmdDrpDnGlrySymbol = 2041,
         cmdBtnDeleteBookmark = 2043,
         cmdDrpDnBtnPageNum = 2056,
         cmdBtnPgNumTopOfPage = 2057,
         cmdBtnPgNumBottomOfPage = 2058,
         cmdBtnPgNumRemove = 2059,

      }

      enum SymbolGalleryButtonCommand : uint
      {
         cmdBtnSymbol_TM = DynamicCommandIdStart + 60,
         cmdBtnSymbol_szLig,
         cmdBtnSymbol_euro,
         cmdBtnSymbol_pound,
         cmdBtnSymbol_yen,
         cmdBtnSymbol_copyright,
         cmdBtnSymbol_registered,
         cmdBtnSymbol_plusMinus,
         cmdBtnSymbol_inequal,
         cmdBtnSymbol_lte,
         cmdBtnSymbol_gte,
         cmdBtnSymbol_division,
         cmdBtnSymbol_multiplication,
         cmdBtnSymbol_infinity,
         cmdBtnSymbol_micro,
         cmdBtnSymbol_alpha,
         cmdBtnSymbol_pi,
         cmdBtnSymbol_omega,
         cmdBtnSymbol_sigma,
         cmdBtnSymbol_smiling,
      }

      enum BreakType
      {
         NextLine,
         Column,
         Page,
         SectionNextPage,
         SectionContinuous,
      }

      #endregion // Enumerations

      #region "  Ribbon event handlers  "

      void BtnTableInsert_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         InsertTable();
      }

      void InsertTable()
      {
         if (_textControl.Tables.Add())
         {
            _fileHandler.DocumentDirty = true;
         }
      }

      void DrpDnGlrySymbol_ItemsSourceReady(object sender, EventArgs e)
      {
         _btnsGallerySymbol = new RibbonButton[SymbolButtonText.Length];
         var commandIDs = Enum.GetValues(typeof(SymbolGalleryButtonCommand));

         for (uint i = 0; i < _btnsGallerySymbol.Length; ++i)
         {
            _btnsGallerySymbol[i] = new RibbonButton(_ribbonMain, (uint)commandIDs.GetValue(i))
            {
               Label = SymbolButtonText[i],
            };
            _btnsGallerySymbol[i].ExecuteEvent += BtnsGallerySymbol_ExecuteEvent;
         }

         IUICollection itemsSource = _drpDnGlrySymbol.ItemsSource;
         itemsSource.Clear();

         foreach (object n in commandIDs)
         {
            itemsSource.Add(new GalleryCommandPropertySet()
            {
               CommandID = (uint)n,
               CommandType = CommandType.Action
            });
         }
      }

      void BtnsGallerySymbol_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         var symb = string.Empty;

         var btn = sender as RibbonButton;
         if (btn == null) return;

         var cmd = (SymbolGalleryButtonCommand)btn.CommandID;

         switch (cmd)
         {
            case SymbolGalleryButtonCommand.cmdBtnSymbol_TM:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.TM];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_szLig:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.SzLig];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_euro:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.Euro];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_pound:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.Pound];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_yen:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.Yen];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_copyright:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.Copyright];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_registered:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.Registered];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_plusMinus:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.PlusMinus];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_inequal:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.Inequal];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_lte:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.Lte];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_gte:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.Gte];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_division:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.Division];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_multiplication:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.Multiplication];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_infinity:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.Infinity];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_micro:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.Micro];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_alpha:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.Alpha];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_pi:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.Pi];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_omega:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.Omega];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_sigma:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.Sigma];
               break;

            case SymbolGalleryButtonCommand.cmdBtnSymbol_smiling:
               symb = SymbolButtonText[(int)SymbolButtonTextIndex.Smiling];
               break;
         }

         _textControl.Focus();

         TXTextControl.IFormattedText textPart = (TXTextControl.IFormattedText)_textControl.TextParts.GetItem();
         if (textPart != null) textPart.Selection.Text = symb;
      }

      void BtnInsertFile_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         try
         {
            _fileHandler.Insert();
         }
         catch (System.Exception ex)
         {
            MessageBox.Show(ex.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
         }
      }

      void TglBtnShowTextFrameMarkerLines_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         var tgl = sender as RibbonToggleButton;
         if (tgl == null) return;

         _textControl.TextFrameMarkerLines = tgl.BooleanValue;
      }

      void BtnInsertTextFrame_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         InsertTextFrame();
      }

      void BtnRemoveFooter_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         RemoveHF(TXTextControl.HeaderFooterType.Footer);
         _btnRemoveFooter.Enabled = FooterExists();
      }

      void BtnEditFooter_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         EditHF(TXTextControl.HeaderFooterType.Footer);
      }

      void BtnRemoveHeader_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         RemoveHF(TXTextControl.HeaderFooterType.Header);
         _btnRemoveHeader.Enabled = HeaderExists();
      }

      void BtnEditHeader_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         EditHF(TXTextControl.HeaderFooterType.Header);
      }

      void TglBtnShowBookmarkMarkers_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         var tgl = sender as RibbonToggleButton;
         if (tgl == null) return;

         _textControl.DocumentTargetMarkers = tgl.BooleanValue;
      }

      void BtnEditBookmark_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         try
         {
            var target = _textControl.DocumentTargets.GetItem();
            if (target == null) return;

            var strTargetName = target.TargetName;

            if (FrmInputBox.ShowInputBox("Target name:", ref strTargetName))
            {
               target.TargetName = strTargetName;
               _fileHandler.DocumentDirty = true;
            }
         }
         catch (Exception ex)
         {
            MessageBox.Show(ex.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
            return;
         }
      }

      void BtnInsertBookmark_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         try
         {
            // Force Exception if standard version:
            _textControl.DocumentTargets.GetItem();

            string strTargetName = "";

            if (FrmInputBox.ShowInputBox("Target Name:", ref strTargetName))
            {
               var dt = new TXTextControl.DocumentTarget(strTargetName) { Deleteable = false };
               if (_textControl.DocumentTargets.Add(dt)) _fileHandler.DocumentDirty = true;
               else MessageBox.Show("Bookmark could not be added.", ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
         }
         catch (Exception ex)
         {
            MessageBox.Show(ex.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
         }
      }

      void BtnDeleteBookmark_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         DeleteBookmark();
      }

      void BtnEditHyperlink_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         try
         {
            // Force exception if standard version:
            _textControl.HypertextLinks.GetItem();

            if (FrmHyperlink.ShowDialog(_textControl) == DialogResult.OK)
            {
               _fileHandler.DocumentDirty = true;
            }
         }
         catch (Exception ex)
         {
            MessageBox.Show(ex.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
         }
      }

      void BtnInsertHyperlink_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         try
         {
            FrmHyperlink.ShowDialog(_textControl);
         }
         catch (Exception ex)
         {
            MessageBox.Show(ex.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
            return;
         }
      }

      void BtnInsertImage_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         InsertImage();
      }

      void BtnPageBreak_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         InsertBreak(BreakType.Page);
      }

      private void InsertBreak(BreakType breakType)
      {
         int dpi = (int)(1440 / _textControl.CreateGraphics().DpiX);

         switch (breakType)
         {
            case BreakType.NextLine:
               _textControl.Selection.Text = "\v";
               break;

            case BreakType.Column:
               _textControl.Selection.Text = "\x0E";
               break;

            case BreakType.Page:
               _textControl.Selection.Text = "\f";
               _textControl.ScrollLocation
                  = new Point(_textControl.InputPosition.Location.X, _textControl.InputPosition.Location.Y - 1440);
               break;

            case BreakType.SectionContinuous:
            case BreakType.SectionNextPage:
               try
               {
                  switch (breakType)
                  {
                     case BreakType.SectionContinuous:
                        _textControl.Sections.Add(TXTextControl.SectionBreakKind.BeginAtNewLine);
                        break;

                     case BreakType.SectionNextPage:
                        _textControl.Sections.Add(TXTextControl.SectionBreakKind.BeginAtNewPage);
                        break;
                  }
               }
               catch (Exception ex)
               {
                  MessageBox.Show(ex.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
                  return;
               }
               break;
         }

         _textControl.ScrollLocation = new Point(_textControl.ScrollLocation.X, (int)(_textControl.InputPosition.Location.Y - (_textControl.Selection.SectionFormat.PageMargins.Top * dpi)));
      }

      void BtnBlankPage_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         if (_textControl.InputPosition.TextPosition == _textControl.TextChars.Count)
         {
            _textControl.Selection.Text = "\f";
         }
         else
         {
            _textControl.Selection.Text = "\f\f";
         }

         _textControl.ScrollLocation
            = new Point(_textControl.InputPosition.Location.X, _textControl.InputPosition.Location.Y - 1440);
      }

      void BtnPgNumRemove_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         RemovePageNumbers();
      }

      void BtnPgNumBottomOfPage_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         InsertPageNumber(HeaderFooterType.Footer);
      }

      void BtnPgNumTopOfPage_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         InsertPageNumber(HeaderFooterType.Header);
      }

      #endregion // Ribbon event handlers

      #region "  Helpers  "

      private void CreateRibbonItems_TabInsert()
      {
         _btnInsertTable = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdBtnInsertTable);
         _drpDnGlrySymbol = new RibbonDropDownGallery(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdDrpDnGlrySymbol);
         _btnBlankPage = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdBtnBlankPage);
         _btnPageBreak = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdBtnPageBreak);
         _btnInsertImage = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdBtnInsertImage);
         _btnInsertHyperlink = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdBtnInsertHyperlink);
         _btnEditHyperlink = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdBtnEditHyperlink);
         _btnInsertBookmark = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdBtnInsertBookmark);
         _btnEditBookmark = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdBtnEditBookmark);
         _tglBtnShowBookmarkMarkers = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdTglBtnShowBookmarkMarkers);
         _btnEditHeader = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdBtnEditHeader);
         _btnRemoveHeader = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdBtnRemoveHeader);
         _btnEditFooter = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdBtnEditFooter);
         _btnRemoveFooter = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdBtnRemoveFooter);
         _btnInsertTextFrame = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdBtnInsertTextFrame);
         _tglBtnShowTextFrameMarkerLines = new RibbonToggleButton(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdTglBtnShowTextFrameMarkerLines);
         _btnInsertFile = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdBtnInsertFile);
         _btnDeleteBookmark = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabInsert.cmdBtnDeleteBookmark);
         _btnPgNumTopOfPage = new RibbonButton(_ribbonMain, (uint) RibbonMarkupCommand_TabInsert.cmdBtnPgNumTopOfPage);
         _btnPgNumBottomOfPage = new RibbonButton(_ribbonMain, (uint) RibbonMarkupCommand_TabInsert.cmdBtnPgNumBottomOfPage);
         _btnPgNumRemove = new RibbonButton(_ribbonMain, (uint) RibbonMarkupCommand_TabInsert.cmdBtnPgNumRemove);

      }

      private void AttachRibbonEvents_TabInsert()
      {
         _btnInsertTable.ExecuteEvent += BtnTableInsert_ExecuteEvent;
         _drpDnGlrySymbol.ItemsSourceReady += DrpDnGlrySymbol_ItemsSourceReady;
         _btnBlankPage.ExecuteEvent += BtnBlankPage_ExecuteEvent;
         _btnPageBreak.ExecuteEvent += BtnPageBreak_ExecuteEvent;
         _btnInsertImage.ExecuteEvent += BtnInsertImage_ExecuteEvent;
         _btnInsertHyperlink.ExecuteEvent += BtnInsertHyperlink_ExecuteEvent;
         _btnEditHyperlink.ExecuteEvent += BtnEditHyperlink_ExecuteEvent;
         _btnInsertBookmark.ExecuteEvent += BtnInsertBookmark_ExecuteEvent;
         _btnEditBookmark.ExecuteEvent += BtnEditBookmark_ExecuteEvent;
         _tglBtnShowBookmarkMarkers.ExecuteEvent += TglBtnShowBookmarkMarkers_ExecuteEvent;
         _btnEditHeader.ExecuteEvent += BtnEditHeader_ExecuteEvent;
         _btnRemoveHeader.ExecuteEvent += BtnRemoveHeader_ExecuteEvent;
         _btnEditFooter.ExecuteEvent += BtnEditFooter_ExecuteEvent;
         _btnRemoveFooter.ExecuteEvent += BtnRemoveFooter_ExecuteEvent;
         _btnInsertTextFrame.ExecuteEvent += BtnInsertTextFrame_ExecuteEvent;
         _tglBtnShowTextFrameMarkerLines.ExecuteEvent += TglBtnShowTextFrameMarkerLines_ExecuteEvent;
         _btnInsertFile.ExecuteEvent += BtnInsertFile_ExecuteEvent;
         _btnDeleteBookmark.ExecuteEvent += BtnDeleteBookmark_ExecuteEvent;
         _btnPgNumTopOfPage.ExecuteEvent += BtnPgNumTopOfPage_ExecuteEvent;
         _btnPgNumBottomOfPage.ExecuteEvent += BtnPgNumBottomOfPage_ExecuteEvent;
         _btnPgNumRemove.ExecuteEvent += BtnPgNumRemove_ExecuteEvent;

      }

      /// <summary>
      /// Inserts an image with text wrapping mode "displace text" at the cursor position.
      /// </summary>
      private void InsertImage()
      {
         var imageNew = new TXTextControl.Image();
         int pos = _textControl.Selection.Start;

         if (_textControl.TextChars.Count > 0
            && _textControl.TextChars[pos] != null
            && _textControl.Paragraphs.GetItem(pos) != null
            && pos < _textControl.TextChars.Count)
         {
            Rectangle rChar = _textControl.TextChars[pos].Bounds;
            Rectangle rPar = _textControl.TextChars[_textControl.Paragraphs.GetItem(pos).Start].Bounds;
            if (_textControl.Paragraphs.GetItem(pos).Start == (pos + 1)) rChar = rPar;
            System.Drawing.Point p
               = new System.Drawing.Point(rChar.Left - rPar.Left + rChar.Width, rChar.Top - rPar.Top);
            _textControl.Images.Add(imageNew, p, -1, TXTextControl.ImageInsertionMode.DisplaceText);
         }
         else
         {
            _textControl.Images.Add(imageNew, TXTextControl.HorizontalAlignment.Left, -1, TXTextControl.ImageInsertionMode.DisplaceText);
         }
      }

      /// <summary>
      /// Inserts a text frame with text wrapping mode "displace text" at the cursor position.
      /// </summary>
      private void InsertTextFrame()
      {
         var sizeTextFrame = new System.Drawing.Size(2268, 2268);   // 4 x 4 cm
         var textFrameNew = new TXTextControl.TextFrame(sizeTextFrame);
         int pos = _textControl.Selection.Start;

         try
         {
            if (_textControl.TextChars.Count > 0
               && _textControl.TextChars[pos] != null
               && _textControl.Paragraphs.GetItem(pos) != null
               && pos < _textControl.TextChars.Count)
            {
               Rectangle rChar = _textControl.TextChars[pos].Bounds;
               Rectangle rPar = _textControl.TextChars[_textControl.Paragraphs.GetItem(pos).Start].Bounds;
               if (_textControl.Paragraphs.GetItem(pos).Start == (pos + 1)) rChar = rPar;
               System.Drawing.Point p
                  = new System.Drawing.Point(rChar.Left - rPar.Left + rChar.Width, rChar.Top - rPar.Top);
               _textControl.TextFrames.Add(textFrameNew, p, -1, TXTextControl.TextFrameInsertionMode.DisplaceText);
            }
            else
            {
               _textControl.TextFrames.Add(textFrameNew, TXTextControl.HorizontalAlignment.Left, -1, TXTextControl.TextFrameInsertionMode.DisplaceText);
            }
         }
         catch (Exception ex)
         {
            MessageBox.Show(ex.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
         }
      }

      private void GoToHF(TXTextControl.HeaderFooterType hfType)
      {
         try
         {
            if (_textControl.Sections.GetItem().HeadersAndFooters.GetItem(hfType) == null)
            {
               _textControl.Sections.GetItem().HeadersAndFooters.Add(hfType);
            }
            _textControl.Sections.GetItem().HeadersAndFooters.GetItem(hfType).Activate();
         }
         catch { }
      }

      private void EditHF(TXTextControl.HeaderFooterType hfType)
      {
         TXTextControl.HeaderFooterType typeFirst;

         switch (hfType)
         {
            case TXTextControl.HeaderFooterType.Header: typeFirst = TXTextControl.HeaderFooterType.FirstPageHeader; break;
            case TXTextControl.HeaderFooterType.Footer: typeFirst = TXTextControl.HeaderFooterType.FirstPageFooter; break;
            default: throw new Exception("Use only TXTextControl.HeaderFooterType.Header or TXTextControl.HeaderFooterType.Footer.");
         }

         try
         {
            // Go to first page header / footer if on first page of section
            if (_textControl.Sections.GetItem().Start == _textControl.InputPosition.Page)
            {
               if (_textControl.Sections.GetItem().HeadersAndFooters.GetItem(typeFirst) != null)
               {
                  _textControl.Sections.GetItem().HeadersAndFooters.GetItem(typeFirst).Activate();
                  return;
               }
            }
         }
         catch (Exception ex)
         {
            MessageBox.Show(ex.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
            return;
         }

         GoToHF(hfType);
      }

      private void RemoveHF(TXTextControl.HeaderFooterType hfType)
      {
         TXTextControl.HeaderFooterType typeFirst;

         switch (hfType)
         {
            case TXTextControl.HeaderFooterType.Header: typeFirst = TXTextControl.HeaderFooterType.FirstPageHeader; break;
            case TXTextControl.HeaderFooterType.Footer: typeFirst = TXTextControl.HeaderFooterType.FirstPageFooter; break;
            default: throw new Exception("Use only TXTextControl.HeaderFooterType.Header or TXTextControl.HeaderFooterType.Footer.");
         }

         try
         {
            // Remove first page header / footer if on first page of section
            if (_textControl.Sections.GetItem().Start == _textControl.InputPosition.Page)
            {
               if (_textControl.Sections.GetItem().HeadersAndFooters.GetItem(typeFirst) != null)
               {
                  _textControl.Sections.GetItem().HeadersAndFooters.Remove(typeFirst);
                  return;
               }
            }

            _textControl.Sections.GetItem().HeadersAndFooters.Remove(hfType);
         }
         catch { }
      }

      private void DeleteBookmark()
      {
         var bookmarks = Bookmarks;
         if (bookmarks.Count == 0)
         {
            MessageBox.Show(string.Format(MsgDocDoesNotContainAny, "bookmarks"), ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
            return;
         }

         var dlg = new DeleteBookmarkDialog(bookmarks, _textControl);
         dlg.ShowDialog();
      }

      /// <summary>
      /// Returns a list of document targets in the current document or null if 
      /// retrieving the document targets led to an exception.
      /// </summary>
      /// <returns>All document targets in the current document.</returns>
      private List<DocumentTarget> Bookmarks
      {
         get
         {
            try
            {
               var result = new List<DocumentTarget>();
               foreach (DocumentTarget tgt in _textControl.DocumentTargets) result.Add(tgt);
               return result;
            }
            catch (Exception exc)
            {
               MessageBox.Show(exc.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }

            return null;
         }
      }

      #endregion // Helpers
   }
}
