﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				Tab_PageLayout.cs
** description:	This file contains event handlers, fields and helper methods concerning the “Page Layout” 
**                ribbon tab.
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Diagnostics;
using System.Drawing;
using System.Reflection;
using System.Windows.Forms;
using RibbonLib;
using RibbonLib.Controls;
using RibbonLib.Controls.Events;
using RibbonLib.Interop;


namespace TX_Text_Control_Words
{
   public partial class FormMain : Form
   {
      #region "  Private fields  "

      private RibbonButton _btnMarginsPaper;
      private RibbonButton _btnPortrait;
      private RibbonButton _btnLandscape;
      private RibbonButton _btnOneColumn;
      private RibbonButton _btnTwoColumns;
      private RibbonButton _btnThreeColumns;
      private RibbonButton _btnMoreColumns;
      private RibbonDropDownGallery _drpDwnGlryBreaks;
      private RibbonButton[] _btnsGalleryBreaks;
      private RibbonButton _btnPageBorders;

      /// <summary>
      /// Label texts of buttons inserting different break kinds.
      /// </summary>
      private static string[] BreakTypeButtonLabelText = 
      {
         "Page", "Column", "Text Wrapping", "Next Page", "Continuous"
      };

      #endregion // Private fields

      #region "  Enumerations  "

      internal enum RibbonMarkupCommand_TabPageLayout : uint
      {
         cmdTabPageLayout = 3000,
         cmdBtnMarginsPaper = 3020,
         cmdDrpDnGlryOrientation = 3021,
         cmdBtnPortrait = 3022,
         cmdBtnLandscape = 3023,
         cmdDrpDnGlryColumns = 3024,
         cmdBtnOneColumn = 3025,
         cmdBtnTwoColumns = 3026,
         cmdBtnThreeColumns = 3027,
         cmdBtnMoreColumns = 3028,
         cmdDrpDnGlryBreaks = 3029,
         cmdBtnPageBorders = 3035,
      }

      internal enum BreakTypeButtonCommand : uint
      {
         cmdBtnBreakPage = DynamicCommandIdStart + 30,
         cmdBtnBreakColumn,
         cmdBtnBreakTextWrap,
         cmdBtnBreakNextPage,
         cmdBtnBreakContinuous,
      }

      #endregion // Enumerations

      #region "  Ribbon events  "

      void DrpDwnGlryBreaks_CategoriesReady(object sender, EventArgs e)
      {
         IUICollection categories = _drpDwnGlryBreaks.Categories;
         categories.Clear();
         categories.Add(new GalleryItemPropertySet() { Label = "Page Breaks", CategoryID = 1 });
         categories.Add(new GalleryItemPropertySet() { Label = "Section Breaks", CategoryID = 2 });
      }

      void DrpDwnGlryBreaks_ItemsSourceReady(object sender, EventArgs e)
      {
         // Create buttons and set images

         _btnsGalleryBreaks = new RibbonButton[_imgLstBreaks.Images.Count];
         Array commandIds = Enum.GetValues(typeof(BreakTypeButtonCommand));

         for (uint i = 0; i < _btnsGalleryBreaks.Length; ++i)
         {
            _btnsGalleryBreaks[i] = new RibbonButton(_ribbonMain, (uint)commandIds.GetValue(i))
            {
               Label = BreakTypeButtonLabelText[(int)i],
               LargeImage = _ribbonMain.ConvertToUIImage((Bitmap)_imgLstBreaks.Images[(int)i])
            };
            _btnsGalleryBreaks[i].ExecuteEvent += BtnsGalleryBreak_ExecuteEvent;
         }

         // Add button commands

         IUICollection itemsSource = _drpDwnGlryBreaks.ItemsSource;
         itemsSource.Clear();

         // Page breaks:

         itemsSource.Add(new GalleryCommandPropertySet()
         {
            CommandID = (uint)BreakTypeButtonCommand.cmdBtnBreakPage,
            CommandType = CommandType.Action,
            CategoryID = 1
         });
         itemsSource.Add(new GalleryCommandPropertySet()
         {
            CommandID = (uint)BreakTypeButtonCommand.cmdBtnBreakColumn,
            CommandType = CommandType.Action,
            CategoryID = 1
         });
         itemsSource.Add(new GalleryCommandPropertySet()
         {
            CommandID = (uint)BreakTypeButtonCommand.cmdBtnBreakTextWrap,
            CommandType = CommandType.Action,
            CategoryID = 1
         });

         // Section breaks:

         itemsSource.Add(new GalleryCommandPropertySet()
         {
            CommandID = (uint)BreakTypeButtonCommand.cmdBtnBreakNextPage,
            CommandType = CommandType.Action,
            CategoryID = 2
         });
         itemsSource.Add(new GalleryCommandPropertySet()
         {
            CommandID = (uint)BreakTypeButtonCommand.cmdBtnBreakContinuous,
            CommandType = CommandType.Action,
            CategoryID = 2
         });
      }

      void BtnsGalleryBreak_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         RibbonButton btn = sender as RibbonButton;
         if (btn == null) return;

         var cmd = (BreakTypeButtonCommand)btn.CommandID;

         switch (cmd)
         {
            // Page breaks:

            case BreakTypeButtonCommand.cmdBtnBreakPage:
               InsertBreak(BreakType.Page);
               break;

            case BreakTypeButtonCommand.cmdBtnBreakColumn:
               InsertBreak(BreakType.Column);
               break;

            case BreakTypeButtonCommand.cmdBtnBreakTextWrap:
               InsertBreak(BreakType.NextLine);
               break;

            // Section breaks:

            case BreakTypeButtonCommand.cmdBtnBreakNextPage:
               InsertBreak(BreakType.SectionNextPage);
               break;

            case BreakTypeButtonCommand.cmdBtnBreakContinuous:
               InsertBreak(BreakType.SectionContinuous);
               break;
         }
      }

      void BtnPageBorders_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         try
         {
            if (_textControl.SectionFormatDialog(3) == System.Windows.Forms.DialogResult.OK)
            {
               _fileHandler.DocumentDirty = true;
            }
         }
         catch (Exception exc)
         {
            MessageBox.Show(exc.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
         }
      }

      void BtnMoreColumns_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         try
         {
            if (_textControl.SectionFormatDialog(2) == System.Windows.Forms.DialogResult.OK)
            {
               _fileHandler.DocumentDirty = true;
            }
         }
         catch (Exception exc)
         {
            MessageBox.Show(exc.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
         }
      }

      void BtnThreeColumns_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         SetColumnCount(3);
      }

      void BtnTwoColumns_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         SetColumnCount(2);
      }

      void BtnOneColumn_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         SetColumnCount(1);
      }

      void BtnLandscape_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         try
         {
            _textControl.Selection.SectionFormat.Landscape = true;
         }
         catch (Exception exc)
         {
            MessageBox.Show(exc.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
         }
      }

      void BtnPortrait_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         try
         {
            _textControl.Selection.SectionFormat.Landscape = false;
         }
         catch (Exception exc)
         {
            MessageBox.Show(exc.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
         }
      }

      void BtnMarginsPaper_ExecuteEvent(object sender, ExecuteEventArgs e)
      {
         try
         {
            if (_textControl.SectionFormatDialog(0) == DialogResult.OK) _fileHandler.DocumentDirty = true;
         }
         catch (Exception ex)
         {
            MessageBox.Show(ex.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
         }
      }

      #endregion // Ribbon events

      #region "  Helpers  "

      private void CreateRibbonItems_TabPageLayout()
      {
         _btnMarginsPaper = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabPageLayout.cmdBtnMarginsPaper);
         _btnPortrait = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabPageLayout.cmdBtnPortrait);
         _btnLandscape = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabPageLayout.cmdBtnLandscape);
         _btnOneColumn = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabPageLayout.cmdBtnOneColumn);
         _btnTwoColumns = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabPageLayout.cmdBtnTwoColumns);
         _btnThreeColumns = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabPageLayout.cmdBtnThreeColumns);
         _btnMoreColumns = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabPageLayout.cmdBtnMoreColumns);
         _drpDwnGlryBreaks = new RibbonDropDownGallery(_ribbonMain, (uint)RibbonMarkupCommand_TabPageLayout.cmdDrpDnGlryBreaks);
         _btnPageBorders = new RibbonButton(_ribbonMain, (uint)RibbonMarkupCommand_TabPageLayout.cmdBtnPageBorders);
      }

      private void AttachRibbonEvents_TabPageLayout()
      {
         _btnMarginsPaper.ExecuteEvent += BtnMarginsPaper_ExecuteEvent;
         _btnPortrait.ExecuteEvent += BtnPortrait_ExecuteEvent;
         _btnLandscape.ExecuteEvent += BtnLandscape_ExecuteEvent;
         _btnOneColumn.ExecuteEvent += BtnOneColumn_ExecuteEvent;
         _btnTwoColumns.ExecuteEvent += BtnTwoColumns_ExecuteEvent;
         _btnThreeColumns.ExecuteEvent += BtnThreeColumns_ExecuteEvent;
         _btnMoreColumns.ExecuteEvent += BtnMoreColumns_ExecuteEvent;
         _drpDwnGlryBreaks.ItemsSourceReady += DrpDwnGlryBreaks_ItemsSourceReady;
         _drpDwnGlryBreaks.CategoriesReady += DrpDwnGlryBreaks_CategoriesReady;
         _btnPageBorders.ExecuteEvent += BtnPageBorders_ExecuteEvent;
      }

      private void SetColumnCount(int columns)
      {
         try
         {
            TXTextControl.Section sect = _textControl.Sections.GetItem();

            if (sect != null)
            {
               sect.Format.EqualColumnWidth = true;
               sect.Format.Columns = columns;
            }
         }
         catch (Exception exc)
         {
            MessageBox.Show(exc.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Information);
         }
      }

      #endregion // Helpers
   }
}
