﻿/*-------------------------------------------------------------------------------------------------------------
** module:			TX Text Control Words
**	file:				TextControlEvents.cs
**
** copyright:		© Text Control GmbH, 1991-2013
** author:			T. Kummerow
**-----------------------------------------------------------------------------------------------------------*/
using System;
using System.Windows.Forms;
using System.Drawing;
using System.Diagnostics;
using System.Reflection;
using TXTextControl.DataVisualization;


namespace TX_Text_Control_Words
{
   public partial class FormMain : Form
   {
      #region "  Event Handlers  "

      void TextControl_GotFocus(object sender, EventArgs e)
      {
         UpdateGUIState();
      }

      private void TextControl_Changed(object sender, EventArgs e)
      {
         _fileHandler.DocumentDirty = true;
         UpdateGUIState();
      }

      private void TextControl_InputPositionChanged(object sender, EventArgs e)
      {
         UpdateGUIState();
      }

      private void TextControl_KeyDown(object sender, KeyEventArgs e)
      {
         HandleKeyDownEvent(e);
      }

      private void TextControl_FormattingStyleListChanged(object sender, EventArgs e)
      {
         FormattingStyleListChanged();
      }

      private void TextControl_ImageDeleted(object sender, TXTextControl.ImageEventArgs e)
      {
         UpdateCurrentObject();
      }

      private void TextControl_TextFrameDeleted(object sender, TXTextControl.TextFrameEventArgs e)
      {
         UpdateCurrentObject();
      }

      private void TextControl_BarcodeDeleted(object sender, TXTextControl.DataVisualization.BarcodeEventArgs e)
      {
         UpdateCurrentObject();
      }

      private void TextControl_ChartDeleted(object sender, TXTextControl.DataVisualization.ChartEventArgs e)
      {
         UpdateCurrentObject();
      }

      private void TextControl_BarcodeSelected(object sender, TXTextControl.DataVisualization.BarcodeEventArgs e)
      {
         UpdateCurrentObject();
      }

      private void TextControl_BarcodeDeselected(object sender, TXTextControl.DataVisualization.BarcodeEventArgs e)
      {
         UpdateCurrentObject();
      }

      private void TextControl_ChartSelected(object sender, TXTextControl.DataVisualization.ChartEventArgs e)
      {
         UpdateCurrentObject();
      }

      private void TextControl_ChartDeselected(object sender, TXTextControl.DataVisualization.ChartEventArgs e)
      {
         UpdateCurrentObject();
      }

      private void TextControl_ImageSelected(object sender, TXTextControl.ImageEventArgs e)
      {
         UpdateCurrentObject();
      }

      private void TextControl_ImageDeselected(object sender, TXTextControl.ImageEventArgs e)
      {
         UpdateCurrentObject();
      }

      private void TextControl_TextFrameSelected(object sender, TXTextControl.TextFrameEventArgs e)
      {
         UpdateCurrentObject();
      }

      private void TextControl_TextFrameDeselected(object sender, TXTextControl.TextFrameEventArgs e)
      {
         UpdateCurrentObject();
      }

      private void TextControl_TextContextMenuOpening(object sender, TXTextControl.TextContextMenuEventArgs e)
      {
          if ((e.ContextMenuLocation & TXTextControl.ContextMenuLocation.TextField) != 0)
          {
              var field = _textControl.ApplicationFields.GetItem();
              if (field == null) return;

              _contextMenuApplicationFields.Show(this, PointToClient(Cursor.Position));
              e.Cancel = true;
          }
          else
          {
              ContextMenuStrip cm = e.TextContextMenu;

              //Get the RS items
              ToolStripItem[] suggestions = rapidSpellDialog1.GetSuggestionsToolStripItems();

              //if there are suggestions (ie if the click was on a spelling error), show them
              if (suggestions != null)
              {
                  cm.Items.Add(new ToolStripSeparator());
                  cm.Items.AddRange(suggestions);
              }
          }
      }

      private void TextControl_DocumentLinkClicked(object sender, TXTextControl.DocumentLinkEventArgs e)
      {
         e.DocumentLink.DocumentTarget.ScrollTo();
      }

      private void TextControl_HypertextLinkClicked(object sender, TXTextControl.HypertextLinkEventArgs e)
      {
         OpenHyperLink(e.HypertextLink.Target);
      }

      private void RulerBarVert_DoubleClick(object sender, EventArgs e)
      {
         _textControl.TabDialog();
      }

      private void TextControl_DragDrop(object sender, DragEventArgs e)
      {
         if (_fileDragDrop.CanDrop)
         {
            switch (_fileDragDrop.FileType)
            {
               case FileDragDropHandler.DraggedFileType.Document:
                  OpenDroppedDocument();
                  break;

               case FileDragDropHandler.DraggedFileType.Image:
                  InsertDroppedImage(e);
                  break;
            }
         }
      }

      private void TextControl_DragEnter(object sender, DragEventArgs e)
      {
         _fileDragDrop.Reset();
         _fileDragDrop.CheckDraggedFiles((string[]) e.Data.GetData(DataFormats.FileDrop));
      }

      private void TextControl_DragOver(object sender, DragEventArgs e)
      {
         if (_fileDragDrop.CanDrop)
         {
            e.Effect = _fileDragDrop.GetDragDropEffect(e.AllowedEffect);
         }
      }

      private void TextControl_Zoomed(object sender, EventArgs e)
      {
         UpdateZoomCheckBoxes();
      }

      #endregion // Event Handlers

      #region "  Helpers  "

      private void HandleKeyDownEvent(KeyEventArgs e)
      {
         switch (e.KeyCode)
         {
            case Keys.Insert:
               // Toggle insertion mode
               if (e.Control || e.Alt || e.Shift) break;
               _textControl.InsertionMode
                  = _textControl.InsertionMode == TXTextControl.InsertionMode.Insert
                  ? TXTextControl.InsertionMode.Overwrite
                  : TXTextControl.InsertionMode.Insert;
               break;

            case Keys.A:
               if (!e.Control || e.Alt || e.Shift) break;
               // Ctrl-A: Select all
               _textControl.SelectAll();
               break;

            case Keys.S:
               if (!e.Control || e.Alt || e.Shift) break;
               // Ctrl-S: save
               try
               {
                  _fileHandler.FileSave();
               }
               catch (Exception exc)
               {
                  MessageBox.Show(exc.Message, "Error saving document", MessageBoxButtons.OK, MessageBoxIcon.Error);
               }
               break;

            case Keys.O:
               if (!e.Control || e.Alt || e.Shift) break;

               // Ctrl-O: open
               if (_fileHandler.DocumentDirty && !FileSaveQuestion()) return;

               _fileHandler.DocumentFileName = string.Empty;
               _fileHandler.FileOpen();
               break;

            case Keys.F:
               if (!e.Control || e.Alt || e.Shift) break;

               // Ctrl-F: search
               _textControl.Find();
               break;
         }
      }

      private bool FileSaveQuestion()
      {
         DialogResult resultMsgBox;

         resultMsgBox = MessageBox.Show("Save changes to " + _fileHandler.DocumentTitle + "?",
                                          "Save Changes?", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
         switch (resultMsgBox)
         {
            case DialogResult.Yes:
               _fileHandler.FileSave();
               if (_fileHandler.DocumentFileName == "") return false;
               break;

            case DialogResult.Cancel:
               return false;
         }

         return true;
      }

      private void OpenHyperLink(string strTarget)
      {
         if (strTarget == "") return;
         else if (!strTarget.StartsWith("http") && !strTarget.StartsWith("ftp") && !strTarget.StartsWith("mailto"))
         {
            strTarget = "http://" + strTarget;
         }

         try
         {
            System.Diagnostics.Process.Start(strTarget);
         }
         catch (Exception ex)
         {
            string msg = ex.Message;
            if (!msg.EndsWith(".")) msg += ".";
            MessageBox.Show("Could not open hyperlink. " + msg, "Hyperlink", MessageBoxButtons.OK, MessageBoxIcon.Error);
         }
      }

      private void OpenDroppedDocument()
      {
         if (_fileHandler.DocumentDirty)
         {
            DialogResult result = MessageBox.Show("Save changes to " + _fileHandler.DocumentTitle + "?", "Save Changes?", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
            if (result == DialogResult.Yes)
            {
               _fileHandler.FileSave();
            }
            else if (result == DialogResult.Cancel) return;
         }
         _fileHandler.DocumentFileName = _fileDragDrop.FileName;
         _fileHandler.FileOpen();
      }

      private void InsertDroppedImage(DragEventArgs e)
      {
         try
         {
            // Get pixel position of mouse cursor inside Text Control
            Point posCursor = _textControl.PointToClient(Cursor.Position);

            // Get bounding rectangle of the first character of the paragraph
            // the image was dropped over
            TXTextControl.Paragraph par = _textControl.Paragraphs.GetItem(posCursor);
            TXTextControl.TextChar charParStart = _textControl.TextChars[par.Start];
            Rectangle rPar = (charParStart != null) ? charParStart.Bounds : new Rectangle();

            // Get bounding rectangle of the character the image was dropped over
            TXTextControl.TextChar txChar = _textControl.TextChars.GetItem(posCursor, true);
            Rectangle rChar = (txChar != null) ? txChar.Bounds : new Rectangle();

            // Calculate image position relative to paragraph position
            var posImg = new Point(rChar.Left - rPar.Left + rChar.Width, rChar.Top - rPar.Top);

            // Insert image anchored to paragraph
            var txImg = new TXTextControl.Image() { FileName = _fileDragDrop.FileName };
            _textControl.Images.Add(txImg, posImg, par.Start, TXTextControl.ImageInsertionMode.DisplaceText);
         }
         catch (Exception exc)
         {
            MessageBox.Show(exc.Message, ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
         }
      }

      #endregion // Helpers
   }
}
